@@ -15,11 +15,14 @@ matplotlib.rcParams['font.size'] = 18
 
 images = ('hubble_deep_field',
           'immunohistochemistry',
+          'lily',
           'microaneurysms',
           'moon',
           'retina',
           'shepp_logan_phantom',
+          'skin',
           'cell',
+          'human_mitosis',
           )
 
 
