@@ -164,7 +164,7 @@ plt.show()
 # different rules of removal can speed up skeletonization and result in
 # different final skeletons.
 #
-# The `thin` function also takes an optional `max_iter` keyword argument to
+# The `thin` function also takes an optional `max_num_iter` keyword argument to
 # limit the number of thinning iterations, and thus produce a relatively
 # thicker skeleton.
 
@@ -172,7 +172,7 @@ from skimage.morphology import skeletonize, thin
 
 skeleton = skeletonize(image)
 thinned = thin(image)
-thinned_partial = thin(image, max_iter=25)
+thinned_partial = thin(image, max_num_iter=25)
 
 fig, axes = plt.subplots(2, 2, figsize=(8, 8), sharex=True, sharey=True)
 ax = axes.ravel()
