@@ -20,7 +20,6 @@ a (9, 9) rectangle and divide it into (3, 3) block, upon which we then apply
 MB-LBP.
 
 """
-from __future__ import print_function
 from skimage.feature import multiblock_lbp
 import numpy as np
 from numpy.testing import assert_equal
@@ -61,7 +60,7 @@ img = draw_multiblock_lbp(test_img, 0, 0, 90, 90,
                           lbp_code=lbp_code, alpha=0.5)
 
 
-plt.imshow(img, interpolation='nearest')
+plt.imshow(img)
 
 plt.show()
 
