@@ -57,33 +57,33 @@ fig, ax = plt.subplots(nrows=2, ncols=4, figsize=(8, 5),
 plt.gray()
 
 # Estimate the average noise standard deviation across color channels.
-sigma_est = estimate_sigma(noisy, multichannel=True, average_sigmas=True)
+sigma_est = estimate_sigma(noisy, channel_axis=-1, average_sigmas=True)
 # Due to clipping in random_noise, the estimate will be a bit smaller than the
 # specified sigma.
-print(f"Estimated Gaussian noise standard deviation = {sigma_est}")
+print(f'Estimated Gaussian noise standard deviation = {sigma_est}')
 
 ax[0, 0].imshow(noisy)
 ax[0, 0].axis('off')
 ax[0, 0].set_title('Noisy')
-ax[0, 1].imshow(denoise_tv_chambolle(noisy, weight=0.1, multichannel=True))
+ax[0, 1].imshow(denoise_tv_chambolle(noisy, weight=0.1, channel_axis=-1))
 ax[0, 1].axis('off')
 ax[0, 1].set_title('TV')
 ax[0, 2].imshow(denoise_bilateral(noisy, sigma_color=0.05, sigma_spatial=15,
-                multichannel=True))
+                channel_axis=-1))
 ax[0, 2].axis('off')
 ax[0, 2].set_title('Bilateral')
-ax[0, 3].imshow(denoise_wavelet(noisy, multichannel=True, rescale_sigma=True))
+ax[0, 3].imshow(denoise_wavelet(noisy, channel_axis=-1, rescale_sigma=True))
 ax[0, 3].axis('off')
 ax[0, 3].set_title('Wavelet denoising')
 
-ax[1, 1].imshow(denoise_tv_chambolle(noisy, weight=0.2, multichannel=True))
+ax[1, 1].imshow(denoise_tv_chambolle(noisy, weight=0.2, channel_axis=-1))
 ax[1, 1].axis('off')
 ax[1, 1].set_title('(more) TV')
 ax[1, 2].imshow(denoise_bilateral(noisy, sigma_color=0.1, sigma_spatial=15,
-                multichannel=True))
+                channel_axis=-1))
 ax[1, 2].axis('off')
 ax[1, 2].set_title('(more) Bilateral')
-ax[1, 3].imshow(denoise_wavelet(noisy, multichannel=True, convert2ycbcr=True,
+ax[1, 3].imshow(denoise_wavelet(noisy, channel_axis=-1, convert2ycbcr=True,
                                 rescale_sigma=True))
 ax[1, 3].axis('off')
 ax[1, 3].set_title('Wavelet denoising\nin YCbCr colorspace')
