@@ -18,7 +18,7 @@ than 0.
 
 import numpy as np
 from skimage import data, util, filters, color
-from skimage.morphology import watershed
+from skimage.segmentation import watershed
 import matplotlib.pyplot as plt
 
 coins = data.coins()
@@ -34,10 +34,10 @@ w1 = watershed(edges, seeds, compactness=0.01)
 
 fig, (ax0, ax1) = plt.subplots(1, 2)
 
-ax0.imshow(color.label2rgb(w0, coins))
+ax0.imshow(color.label2rgb(w0, coins, bg_label=-1))
 ax0.set_title('Classical watershed')
 
-ax1.imshow(color.label2rgb(w1, coins))
+ax1.imshow(color.label2rgb(w1, coins, bg_label=-1))
 ax1.set_title('Compact watershed')
 
 plt.show()
