@@ -4,11 +4,11 @@
 # stdlib imports
 import os, sys
 
+from packaging import version as _version
+
 # local imports
 from apigen import ApiDocWriter
 
-# version comparison
-from distutils.version import LooseVersion as V
 
 #*****************************************************************************
 
@@ -35,13 +35,14 @@ if __name__ == '__main__':
     # are not (re)generated. This avoids automatic generation of documentation
     # for older or newer versions if such versions are installed on the system.
 
-    installed_version = V(module.__version__)
+    # exclude any appended git hash and date
+    installed_version = _version.parse(module.__version__.split('+git')[0])
 
     source_lines = open('../skimage/__init__.py').readlines()
     version = 'vUndefined'
     for l in source_lines:
         if l.startswith('__version__'):
-            source_version = V(l.split("'")[1])
+            source_version = _version.parse(l.split("'")[1])
             break
 
     if source_version != installed_version:
@@ -49,10 +50,12 @@ if __name__ == '__main__':
 
     outdir = 'source/api'
     docwriter = ApiDocWriter(package)
-    docwriter.package_skip_patterns += [r'\.fixes$',
-                                        r'\.externals$',
-                                        r'filter$',
-                                        ]
+    docwriter.package_skip_patterns += [
+        r'\.fixes$',
+        r'\.externals$',
+        r'filter$',
+        r'viewer.viewers$',  # all functions already imported to viewer
+    ]
     docwriter.write_api_docs(outdir)
     docwriter.write_index(outdir, 'api', relative_to='source/api')
     print('%d files written' % len(docwriter.written_modules))
