@@ -2,7 +2,10 @@ __all__ = ['polygon_clip', 'polygon_area']
 
 import numpy as np
 
+from .version_requirements import require
 
+
+@require("matplotlib", ">=3.0.3")
 def polygon_clip(rp, cp, r0, c0, r1, c1):
     """Clip a polygon to the given bounding box.
 
