@@ -6,11 +6,9 @@ from .colorconv import (convert_colorspace,
                         xyz2rgb,
                         rgb2rgbcie,
                         rgbcie2rgb,
-                        rgb2grey,
                         rgb2gray,
                         gray2rgb,
                         gray2rgba,
-                        grey2rgb,
                         xyz2lab,
                         lab2xyz,
                         lab2rgb,
@@ -75,11 +73,9 @@ __all__ = ['convert_colorspace',
            'xyz2rgb',
            'rgb2rgbcie',
            'rgbcie2rgb',
-           'rgb2grey',
            'rgb2gray',
            'gray2rgb',
            'gray2rgba',
-           'grey2rgb',
            'xyz2lab',
            'lab2xyz',
            'lab2rgb',
