@@ -2,10 +2,10 @@ import itertools
 
 import numpy as np
 
-from .._shared.utils import warn, change_default_value
+from .._shared.utils import _supported_float_type, warn
 from ..util import img_as_float
 from . import rgb_colors
-from .colorconv import rgb2gray, gray2rgb
+from .colorconv import gray2rgb, rgb2hsv, hsv2rgb
 
 
 __all__ = ['color_dict', 'label2rgb', 'DEFAULT_COLORS']
@@ -71,18 +71,19 @@ def _match_label_with_color(label, colors, bg_label, bg_color):
     return mapped_labels, color_cycle
 
 
-@change_default_value("bg_label", new_value=0, changed_version="0.19")
 def label2rgb(label, image=None, colors=None, alpha=0.3,
-              bg_label=-1, bg_color=(0, 0, 0), image_alpha=1, kind='overlay'):
+              bg_label=0, bg_color=(0, 0, 0), image_alpha=1, kind='overlay',
+              *, saturation=0, channel_axis=-1):
     """Return an RGB image where color-coded labels are painted over the image.
 
     Parameters
     ----------
-    label : array, shape (M, N)
+    label : ndarray
         Integer array of labels with the same shape as `image`.
-    image : array, shape (M, N, 3), optional
-        Image used as underlay for labels. If the input is an RGB image, it's
-        converted to grayscale before coloring.
+    image : ndarray, optional
+        Image used as underlay for labels. It should have the same shape as
+        `labels`, optionally with an additional RGB (channels) axis. If `image`
+        is an RGB image, it is converted to grayscale before coloring.
     colors : list, optional
         List of colors. If the number of labels exceeds the number of colors,
         then the colors are cycled.
@@ -102,33 +103,50 @@ def label2rgb(label, image=None, colors=None, alpha=0.3,
         and overlays the colored labels over the original image. 'avg' replaces
         each labeled segment with its average color, for a stained-class or
         pastel painting appearance.
+    saturation : float [0, 1], optional
+        Parameter to control the saturation applied to the original image
+        between fully saturated (original RGB, `saturation=1`) and fully
+        unsaturated (grayscale, `saturation=0`). Only applies when
+        `kind='overlay'`.
+    channel_axis : int, optional
+        This parameter indicates which axis of the output array will correspond
+        to channels. If `image` is provided, this must also match the axis of
+        `image` that corresponds to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Returns
     -------
-    result : array of float, shape (M, N, 3)
+    result : ndarray of float, same shape as `image`
         The result of blending a cycling colormap (`colors`) for each distinct
         value in `label` with the image, at a certain alpha value.
     """
+    if image is not None:
+        image = np.moveaxis(image, source=channel_axis, destination=-1)
     if kind == 'overlay':
-        return _label2rgb_overlay(label, image, colors, alpha, bg_label,
-                                  bg_color, image_alpha)
+        rgb = _label2rgb_overlay(label, image, colors, alpha, bg_label,
+                                 bg_color, image_alpha, saturation)
     elif kind == 'avg':
-        return _label2rgb_avg(label, image, bg_label, bg_color)
+        rgb = _label2rgb_avg(label, image, bg_label, bg_color)
     else:
         raise ValueError("`kind` must be either 'overlay' or 'avg'.")
+    return np.moveaxis(rgb, source=-1, destination=channel_axis)
 
 
 def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
-                       bg_label=-1, bg_color=None, image_alpha=1):
+                       bg_label=-1, bg_color=None, image_alpha=1,
+                       saturation=0):
     """Return an RGB image where color-coded labels are painted over the image.
 
     Parameters
     ----------
-    label : array, shape (M, N)
+    label : ndarray
         Integer array of labels with the same shape as `image`.
-    image : array, shape (M, N, 3), optional
-        Image used as underlay for labels. If the input is an RGB image, it's
-        converted to grayscale before coloring.
+    image : ndarray, optional
+        Image used as underlay for labels. It should have the same shape as
+        `labels`, optionally with an additional RGB (channels) axis. If `image`
+        is an RGB image, it is converted to grayscale before coloring.
     colors : list, optional
         List of colors. If the number of labels exceeds the number of colors,
         then the colors are cycled.
@@ -142,13 +160,20 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
         between [0, 1].
     image_alpha : float [0, 1], optional
         Opacity of the image.
+    saturation : float [0, 1], optional
+        Parameter to control the saturation applied to the original image
+        between fully saturated (original RGB, `saturation=1`) and fully
+        unsaturated (grayscale, `saturation=0`).
 
     Returns
     -------
-    result : array of float, shape (M, N, 3)
+    result : ndarray of float, same shape as `image`
         The result of blending a cycling colormap (`colors`) for each distinct
         value in `label` with the image, at a certain alpha value.
     """
+    if not 0 <= saturation <= 1:
+        warn(f'saturation must be in range [0, 1], got {saturation}')
+
     if colors is None:
         colors = DEFAULT_COLORS
     colors = [_rgb_vector(c) for c in colors]
@@ -158,17 +183,27 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
         # Opacity doesn't make sense if no image exists.
         alpha = 1
     else:
-        if not image.shape[:2] == label.shape:
+        if (image.shape[:label.ndim] != label.shape
+                or image.ndim > label.ndim + 1):
             raise ValueError("`image` and `label` must be the same shape")
 
+        if image.ndim == label.ndim + 1 and image.shape[-1] != 3:
+            raise ValueError(
+                "`image` must be RGB (image.shape[-1] must be 3)."
+            )
+
         if image.min() < 0:
             warn("Negative intensities in `image` are not supported")
 
+        float_dtype = _supported_float_type(image.dtype)
+        image = img_as_float(image).astype(float_dtype, copy=False)
         if image.ndim > label.ndim:
-            image = img_as_float(rgb2gray(image))
-        else:
-            image = img_as_float(image)
-        image = gray2rgb(image) * image_alpha + (1 - image_alpha)
+            hsv = rgb2hsv(image)
+            hsv[..., 1] *= saturation
+            image = hsv2rgb(hsv)
+        elif image.ndim == label.ndim:
+            image = gray2rgb(image)
+        image = image * image_alpha + (1 - image_alpha)
 
     # Ensure that all labels are non-negative so we can index into
     # `label_to_color` correctly.
@@ -183,12 +218,13 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
     label = label.astype(new_type)
 
     mapped_labels_flat, color_cycle = _match_label_with_color(label, colors,
-                                                              bg_label, bg_color)
+                                                              bg_label,
+                                                              bg_color)
 
     if len(mapped_labels_flat) == 0:
         return image
 
-    dense_labels = range(max(mapped_labels_flat) + 1)
+    dense_labels = range(np.max(mapped_labels_flat) + 1)
 
     label_to_color = np.stack([c for i, c in zip(dense_labels, color_cycle)])
 
@@ -209,7 +245,7 @@ def _label2rgb_avg(label_field, image, bg_label=0, bg_color=(0, 0, 0)):
 
     Parameters
     ----------
-    label_field : array of int
+    label_field : ndarray of int
         A segmentation of an image.
     image : array, shape ``label_field.shape + (3,)``
         A color image of the same spatial shape as `label_field`.
@@ -220,10 +256,10 @@ def _label2rgb_avg(label_field, image, bg_label=0, bg_color=(0, 0, 0)):
 
     Returns
     -------
-    out : array, same shape and type as `image`
+    out : ndarray, same shape and type as `image`
         The output visualization.
     """
-    out = np.zeros(label_field.shape + (3,))
+    out = np.zeros(label_field.shape + (3,), dtype=image.dtype)
     labels = np.unique(label_field)
     bg = (labels == bg_label)
     if bg.any():
