@@ -8,13 +8,16 @@ Authors
 :license: modified BSD
 """
 
+import colorsys
 import numpy as np
 import pytest
-from skimage._shared.testing import assert_equal, assert_almost_equal
-from skimage._shared.testing import assert_array_almost_equal, fetch
-from skimage._shared.testing import TestCase
+from numpy.testing import (assert_almost_equal, assert_array_almost_equal,
+                           assert_equal)
 
-from skimage.util import img_as_float, img_as_ubyte, img_as_float32
+from skimage import data
+from skimage._shared._warnings import expected_warnings
+from skimage._shared.testing import fetch
+from skimage._shared.utils import _supported_float_type, slice_at_axis
 from skimage.color import (rgb2hsv, hsv2rgb,
                            rgb2xyz, xyz2rgb,
                            rgb2hed, hed2rgb,
@@ -34,19 +37,17 @@ from skimage.color import (rgb2hsv, hsv2rgb,
                            rgb2ycbcr, ycbcr2rgb,
                            rgb2ydbdr, ydbdr2rgb,
                            rgba2rgb, gray2rgba)
-
-from skimage._shared._warnings import expected_warnings
-from skimage import data
-import colorsys
+from skimage.util import img_as_float, img_as_ubyte, img_as_float32
 
 
-class TestColorconv(TestCase):
+class TestColorconv():
 
     img_rgb = data.colorwheel()
     img_grayscale = data.camera()
     img_rgba = np.array([[[0, 0.5, 1, 0],
                           [0, 0.5, 1, 1],
                           [0, 0.5, 1, 0.5]]]).astype(float)
+    img_stains = img_as_float(img_rgb) * 0.3
 
     colbars = np.array([[1, 1, 0, 0, 1, 1, 0, 0],
                         [1, 1, 1, 1, 0, 0, 0, 0],
@@ -76,20 +77,37 @@ class TestColorconv(TestCase):
                           ])
 
     # RGBA to RGB
-    def test_rgba2rgb_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, 2, -1, -2, -3])
+    def test_rgba2rgb_conversion(self, channel_axis):
         rgba = self.img_rgba
-        rgb = rgba2rgb(rgba)
+
+        rgba = np.moveaxis(rgba, source=-1, destination=channel_axis)
+        rgb = rgba2rgb(rgba, channel_axis=channel_axis)
+        rgb = np.moveaxis(rgb, source=channel_axis, destination=-1)
+
         expected = np.array([[[1, 1, 1],
                               [0, 0.5, 1],
                               [0.5, 0.75, 1]]]).astype(float)
-        self.assertEqual(rgb.shape, expected.shape)
+        assert_equal(rgb.shape, expected.shape)
         assert_almost_equal(rgb, expected)
 
     def test_rgba2rgb_error_grayscale(self):
-        self.assertRaises(ValueError, rgba2rgb, self.img_grayscale)
+        with pytest.raises(ValueError):
+            rgba2rgb(self.img_grayscale)
+
+    @pytest.mark.parametrize("channel_axis", [None, 1.5])
+    def test_rgba2rgb_error_channel_axis_invalid(self, channel_axis):
+        with pytest.raises(TypeError):
+            rgba2rgb(self.img_rgba, channel_axis=channel_axis)
+
+    @pytest.mark.parametrize("channel_axis", [-4, 3])
+    def test_rgba2rgb_error_channel_axis_out_of_range(self, channel_axis):
+        with pytest.raises(np.AxisError):
+            rgba2rgb(self.img_rgba, channel_axis=channel_axis)
 
     def test_rgba2rgb_error_rgb(self):
-        self.assertRaises(ValueError, rgba2rgb, self.img_rgb)
+        with pytest.raises(ValueError):
+            rgba2rgb(self.img_rgb)
 
     def test_rgba2rgb_dtype(self):
         rgba = self.img_rgba.astype('float64')
@@ -99,9 +117,15 @@ class TestColorconv(TestCase):
         assert rgba2rgb(rgba32).dtype == rgba32.dtype
 
     # RGB to HSV
-    def test_rgb2hsv_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_rgb2hsv_conversion(self, channel_axis):
         rgb = img_as_float(self.img_rgb)[::16, ::16]
-        hsv = rgb2hsv(rgb).reshape(-1, 3)
+
+        _rgb = np.moveaxis(rgb, source=-1, destination=channel_axis)
+        hsv = rgb2hsv(_rgb, channel_axis=channel_axis)
+        hsv = np.moveaxis(hsv, source=channel_axis, destination=-1)
+        hsv = hsv.reshape(-1, 3)
+
         # ground truth from colorsys
         gt = np.array([colorsys.rgb_to_hsv(pt[0], pt[1], pt[2])
                        for pt in rgb.reshape(-1, 3)]
@@ -109,7 +133,8 @@ class TestColorconv(TestCase):
         assert_almost_equal(hsv, gt)
 
     def test_rgb2hsv_error_grayscale(self):
-        self.assertRaises(ValueError, rgb2hsv, self.img_grayscale)
+        with pytest.raises(ValueError):
+            rgb2hsv(self.img_grayscale)
 
     def test_rgb2hsv_dtype(self):
         rgb = img_as_float(self.img_rgb)
@@ -119,17 +144,24 @@ class TestColorconv(TestCase):
         assert rgb2hsv(rgb32).dtype == rgb32.dtype
 
     # HSV to RGB
-    def test_hsv2rgb_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_hsv2rgb_conversion(self, channel_axis):
         rgb = self.img_rgb.astype("float32")[::16, ::16]
         # create HSV image with colorsys
         hsv = np.array([colorsys.rgb_to_hsv(pt[0], pt[1], pt[2])
                         for pt in rgb.reshape(-1, 3)]).reshape(rgb.shape)
+
+        hsv = np.moveaxis(hsv, source=-1, destination=channel_axis)
+        _rgb = hsv2rgb(hsv, channel_axis=channel_axis)
+        _rgb = np.moveaxis(_rgb, source=channel_axis, destination=-1)
+
         # convert back to RGB and compare with original.
         # relative precision for RGB -> HSV roundtrip is about 1e-6
-        assert_almost_equal(rgb, hsv2rgb(hsv), decimal=4)
+        assert_almost_equal(rgb, _rgb, decimal=4)
 
     def test_hsv2rgb_error_grayscale(self):
-        self.assertRaises(ValueError, hsv2rgb, self.img_grayscale)
+        with pytest.raises(ValueError):
+            hsv2rgb(self.img_grayscale)
 
     def test_hsv2rgb_dtype(self):
         rgb = self.img_rgb.astype("float32")[::16, ::16]
@@ -143,7 +175,8 @@ class TestColorconv(TestCase):
         assert hsv2rgb(hsv32).dtype == hsv32.dtype
 
     # RGB to XYZ
-    def test_rgb2xyz_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_rgb2xyz_conversion(self, channel_axis):
         gt = np.array([[[0.950456, 1.      , 1.088754],
                         [0.538003, 0.787329, 1.06942 ],
                         [0.592876, 0.28484 , 0.969561],
@@ -152,12 +185,20 @@ class TestColorconv(TestCase):
                         [0.35758 , 0.71516 , 0.119193],
                         [0.412453, 0.212671, 0.019334],
                         [0.      , 0.      , 0.      ]]])
-        assert_almost_equal(rgb2xyz(self.colbars_array), gt)
+
+        img = np.moveaxis(
+            self.colbars_array, source=-1, destination=channel_axis
+        )
+        out = rgb2xyz(img, channel_axis=channel_axis)
+        out = np.moveaxis(out, source=channel_axis, destination=-1)
+
+        assert_almost_equal(out, gt)
 
     # stop repeating the "raises" checks for all other functions that are
     # implemented with color._convert()
     def test_rgb2xyz_error_grayscale(self):
-        self.assertRaises(ValueError, rgb2xyz, self.img_grayscale)
+        with pytest.raises(ValueError):
+            rgb2xyz(self.img_grayscale)
 
     def test_rgb2xyz_dtype(self):
         img = self.colbars_array
@@ -179,39 +220,58 @@ class TestColorconv(TestCase):
         assert xyz2rgb(img32).dtype == img32.dtype
 
     # RGB<->XYZ roundtrip on another image
-    def test_xyz_rgb_roundtrip(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_xyz_rgb_roundtrip(self, channel_axis):
         img_rgb = img_as_float(self.img_rgb)
-        assert_array_almost_equal(xyz2rgb(rgb2xyz(img_rgb)), img_rgb)
 
-    # RGB<->HED roundtrip with ubyte image
+        img_rgb = np.moveaxis(img_rgb, source=-1, destination=channel_axis)
+        round_trip = xyz2rgb(rgb2xyz(img_rgb, channel_axis=channel_axis),
+                             channel_axis=channel_axis)
+
+        assert_array_almost_equal(round_trip, img_rgb)
+
+    # HED<->RGB roundtrip with ubyte image
     def test_hed_rgb_roundtrip(self):
-        img_rgb = img_as_ubyte(self.img_rgb)
-        new = img_as_ubyte(hed2rgb(rgb2hed(img_rgb)))
-        assert_equal(new, img_rgb)
+        img_in = img_as_ubyte(self.img_stains)
+        img_out = rgb2hed(hed2rgb(img_in))
+        assert_equal(img_as_ubyte(img_out), img_in)
 
-    # RGB<->HED roundtrip with float image
-    def test_hed_rgb_float_roundtrip(self):
-        img_rgb = img_as_float(self.img_rgb)
-        assert_array_almost_equal(hed2rgb(rgb2hed(img_rgb)), img_rgb)
+    # HED<->RGB roundtrip with float image
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_hed_rgb_float_roundtrip(self, channel_axis):
+        img_in = self.img_stains
+        img_in = np.moveaxis(img_in, source=-1, destination=channel_axis)
+        img_out = rgb2hed(
+            hed2rgb(img_in, channel_axis=channel_axis),
+            channel_axis=channel_axis
+        )
+        assert_array_almost_equal(img_out, img_in)
 
-    # RGB<->HDX roundtrip with ubyte image
-    def test_hdx_rgb_roundtrip(self):
-        from skimage.color.colorconv import hdx_from_rgb, rgb_from_hdx
-        img_rgb = self.img_rgb
-        conv = combine_stains(separate_stains(img_rgb, hdx_from_rgb),
-                              rgb_from_hdx)
-        assert_equal(img_as_ubyte(conv), img_rgb)
+    # BRO<->RGB roundtrip with ubyte image
+    def test_bro_rgb_roundtrip(self):
+        from skimage.color.colorconv import bro_from_rgb, rgb_from_bro
+        img_in = img_as_ubyte(self.img_stains)
+        img_out = combine_stains(img_in, rgb_from_bro)
+        img_out = separate_stains(img_out, bro_from_rgb)
+        assert_equal(img_as_ubyte(img_out), img_in)
 
-    # RGB<->HDX roundtrip with float image
-    def test_hdx_rgb_roundtrip_float(self):
-        from skimage.color.colorconv import hdx_from_rgb, rgb_from_hdx
-        img_rgb = img_as_float(self.img_rgb)
-        conv = combine_stains(separate_stains(img_rgb, hdx_from_rgb),
-                              rgb_from_hdx)
-        assert_array_almost_equal(conv, img_rgb)
+    # BRO<->RGB roundtrip with float image
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1])
+    def test_bro_rgb_roundtrip_float(self, channel_axis):
+        from skimage.color.colorconv import bro_from_rgb, rgb_from_bro
+        img_in = self.img_stains
+        img_in = np.moveaxis(img_in, source=-1, destination=channel_axis)
+        img_out = combine_stains(
+            img_in, rgb_from_bro, channel_axis=channel_axis
+        )
+        img_out = separate_stains(
+            img_out, bro_from_rgb, channel_axis=channel_axis
+        )
+        assert_array_almost_equal(img_out, img_in)
 
     # RGB to RGB CIE
-    def test_rgb2rgbcie_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_rgb2rgbcie_conversion(self, channel_axis):
         gt = np.array([[[ 0.1488856 ,  0.18288098,  0.19277574],
                         [ 0.01163224,  0.16649536,  0.18948516],
                         [ 0.12259182,  0.03308008,  0.17298223],
@@ -220,7 +280,15 @@ class TestColorconv(TestCase):
                         [ 0.02629378,  0.1498009 ,  0.01979351],
                         [ 0.13725336,  0.01638562,  0.00329059],
                         [ 0.        ,  0.        ,  0.        ]]])
-        assert_almost_equal(rgb2rgbcie(self.colbars_array), gt)
+
+        img = np.moveaxis(
+            self.colbars_array, source=-1, destination=channel_axis
+        )
+        out = rgb2rgbcie(img, channel_axis=channel_axis)
+
+        out = np.moveaxis(out, source=channel_axis, destination=-1)
+
+        assert_almost_equal(out, gt)
 
     def test_rgb2rgbcie_dtype(self):
         img = self.colbars_array.astype('float64')
@@ -230,10 +298,15 @@ class TestColorconv(TestCase):
         assert rgb2rgbcie(img32).dtype == img32.dtype
 
     # RGB CIE to RGB
-    def test_rgbcie2rgb_conversion(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_rgbcie2rgb_conversion(self, channel_axis):
+        rgb = np.moveaxis(
+            self.colbars_array, source=-1, destination=channel_axis
+        )
+        round_trip = rgbcie2rgb(rgb2rgbcie(rgb, channel_axis=channel_axis),
+                                channel_axis=channel_axis)
         # only roundtrip test, we checked rgb2rgbcie above already
-        assert_almost_equal(rgbcie2rgb(rgb2rgbcie(self.colbars_array)),
-                            self.colbars_array)
+        assert_almost_equal(round_trip, rgb)
 
     def test_rgbcie2rgb_dtype(self):
         img = rgb2rgbcie(self.colbars_array).astype('float64')
@@ -242,7 +315,8 @@ class TestColorconv(TestCase):
         assert rgbcie2rgb(img).dtype == img.dtype
         assert rgbcie2rgb(img32).dtype == img32.dtype
 
-    def test_convert_colorspace(self):
+    @pytest.mark.parametrize("channel_axis", [0, -1])
+    def test_convert_colorspace(self, channel_axis):
         colspaces = ['HSV', 'RGB CIE', 'XYZ', 'YCbCr', 'YPbPr', 'YDbDr']
         colfuncs_from = [
             hsv2rgb, rgbcie2rgb, xyz2rgb,
@@ -253,26 +327,34 @@ class TestColorconv(TestCase):
             rgb2ycbcr, rgb2ypbpr, rgb2ydbdr
         ]
 
+        colbars_array = np.moveaxis(
+            self.colbars_array, source=-1, destination=channel_axis
+        )
+
+        kw = dict(channel_axis=channel_axis)
+
         assert_almost_equal(
-            convert_colorspace(self.colbars_array, 'RGB', 'RGB'),
-            self.colbars_array)
+            convert_colorspace(colbars_array, 'RGB', 'RGB', **kw),
+            colbars_array)
 
         for i, space in enumerate(colspaces):
-            gt = colfuncs_from[i](self.colbars_array)
+            gt = colfuncs_from[i](colbars_array, **kw)
             assert_almost_equal(
-                convert_colorspace(self.colbars_array, space, 'RGB'), gt)
-            gt = colfuncs_to[i](self.colbars_array)
+                convert_colorspace(colbars_array, space, 'RGB', **kw), gt)
+            gt = colfuncs_to[i](colbars_array, **kw)
             assert_almost_equal(
-                convert_colorspace(self.colbars_array, 'RGB', space), gt)
+                convert_colorspace(colbars_array, 'RGB', space, **kw), gt)
 
-        self.assertRaises(ValueError, convert_colorspace,
-                          self.colbars_array, 'nokey', 'XYZ')
-        self.assertRaises(ValueError, convert_colorspace,
-                          self.colbars_array, 'RGB', 'nokey')
+        with pytest.raises(ValueError):
+            convert_colorspace(self.colbars_array, 'nokey', 'XYZ')
+        with pytest.raises(ValueError):
+            convert_colorspace(self.colbars_array, 'RGB', 'nokey')
 
-    def test_rgb2gray(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_rgb2gray(self, channel_axis):
         x = np.array([1, 1, 1]).reshape((1, 1, 3)).astype(float)
-        g = rgb2gray(x)
+        x = np.moveaxis(x, source=-1, destination=channel_axis)
+        g = rgb2gray(x, channel_axis=channel_axis)
         assert_array_almost_equal(g, 1)
 
         assert_equal(g.shape, (1, 1))
@@ -283,13 +365,13 @@ class TestColorconv(TestCase):
         assert rgb2gray(x[:5, :5]).flags["C_CONTIGUOUS"]
 
     def test_rgb2gray_alpha(self):
-        x = np.random.rand(10, 10, 4)
-        with expected_warnings(['Non RGB image conversion']):
-            assert rgb2gray(x).ndim == 2
+        x = np.empty((10, 10, 4))
+        with pytest.raises(ValueError):
+            rgb2gray(x)
 
     def test_rgb2gray_on_gray(self):
-        with expected_warnings(['The behavior of rgb2gray will change']):
-            rgb2gray(np.random.rand(5, 5))
+        with pytest.raises(ValueError):
+            rgb2gray(np.empty((5, 5)))
 
     def test_rgb2gray_dtype(self):
         img = np.random.rand(10, 10, 3).astype('float64')
@@ -306,20 +388,30 @@ class TestColorconv(TestCase):
                                   self.lab_array, decimal=3)
 
         # Test the conversion with the rest of the illuminants.
-        for I in ["d50", "d55", "d65", "d75"]:
-            for obs in ["2", "10"]:
-                fname = "color/tests/data/lab_array_{0}_{1}.npy".format(I, obs)
+        for I in ["A", "B", "C", "d50", "d55", "d65"]:
+            I = I.lower()
+            for obs in ["2", "10", "R"]:
+                obs = obs.lower()
+                fname = f'color/tests/data/lab_array_{I}_{obs}.npy'
                 lab_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(lab_array_I_obs,
                                           xyz2lab(self.xyz_array, I, obs),
                                           decimal=2)
-        for I in ["a", "e"]:
-            fname = "color/tests/data/lab_array_{0}_2.npy".format(I)
+        for I in ["d75", "e"]:
+            fname = f'color/tests/data/lab_array_{I}_2.npy'
             lab_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(lab_array_I_obs,
                                       xyz2lab(self.xyz_array, I, "2"),
                                       decimal=2)
 
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_xyz2lab_channel_axis(self, channel_axis):
+        # test conversion with channels along a specified axis
+        xyz = np.moveaxis(self.xyz_array, source=-1, destination=channel_axis)
+        lab = xyz2lab(xyz, channel_axis=channel_axis)
+        lab = np.moveaxis(lab, source=channel_axis, destination=-1)
+        assert_array_almost_equal(lab, self.lab_array, decimal=3)
+
     def test_xyz2lab_dtype(self):
         img = self.xyz_array.astype('float64')
         img32 = img.astype('float32')
@@ -332,28 +424,34 @@ class TestColorconv(TestCase):
                                   self.xyz_array, decimal=3)
 
         # Test the conversion with the rest of the illuminants.
-        for I in ["d50", "d55", "d65", "d75"]:
-            for obs in ["2", "10"]:
-                fname = "color/tests/data/lab_array_{0}_{1}.npy".format(I, obs)
+        for I in ["A", "B", "C", "d50", "d55", "d65"]:
+            I = I.lower()
+            for obs in ["2", "10", "R"]:
+                obs = obs.lower()
+                fname = f'color/tests/data/lab_array_{I}_{obs}.npy'
                 lab_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, obs),
                                           self.xyz_array, decimal=3)
-        for I in ["a", "e"]:
-            fname = "lab_array_{0}_2.npy".format(I, obs)
-            lab_array_I_obs = np.load(fetch('color/tests/data/' + fname))
+        for I in ["d75", "e"]:
+            fname = f'color/tests/data/lab_array_{I}_2.npy'
+            lab_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, "2"),
                                       self.xyz_array, decimal=3)
 
         # And we include a call to test the exception handling in the code.
-        try:
-            xs = lab2xyz(lab_array_I_obs, "NaI", "2")   # Not an illuminant
-        except ValueError:
-            pass
+        with pytest.raises(ValueError):
+            lab2xyz(lab_array_I_obs, "NaI", "2")   # Not an illuminant
 
-        try:
-            xs = lab2xyz(lab_array_I_obs, "d50", "42")   # Not a degree
-        except ValueError:
-            pass
+        with pytest.raises(ValueError):
+            lab2xyz(lab_array_I_obs, "d50", "42")   # Not a degree
+
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_lab2xyz_channel_axis(self, channel_axis):
+        # test conversion with channels along a specified axis
+        lab = np.moveaxis(self.lab_array, source=-1, destination=channel_axis)
+        xyz = lab2xyz(lab, channel_axis=channel_axis)
+        xyz = np.moveaxis(xyz, source=channel_axis, destination=-1)
+        assert_array_almost_equal(xyz, self.xyz_array, decimal=3)
 
     def test_lab2xyz_dtype(self):
         img = self.lab_array.astype('float64')
@@ -370,20 +468,30 @@ class TestColorconv(TestCase):
         """
         # Obtained with D65 white point, sRGB model and gamma
         gt_for_colbars = np.array([
-            [100,0,0],
+            [100, 0, 0],
             [97.1393, -21.5537, 94.4780],
             [91.1132, -48.0875, -14.1312],
             [87.7347, -86.1827, 83.1793],
             [60.3242, 98.2343, -60.8249],
             [53.2408, 80.0925, 67.2032],
             [32.2970, 79.1875, -107.8602],
-            [0,0,0]]).T
+            [0, 0, 0]]).T
         gt_array = np.swapaxes(gt_for_colbars.reshape(3, 4, 2), 0, 2)
-        assert_array_almost_equal(rgb2lab(self.colbars_array), gt_array, decimal=2)
+        assert_array_almost_equal(
+            rgb2lab(self.colbars_array), gt_array, decimal=2
+        )
 
-    def test_lab_rgb_roundtrip(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_lab_rgb_roundtrip(self, channel_axis):
         img_rgb = img_as_float(self.img_rgb)
-        assert_array_almost_equal(lab2rgb(rgb2lab(img_rgb)), img_rgb)
+        img_rgb = np.moveaxis(img_rgb, source=-1, destination=channel_axis)
+        assert_array_almost_equal(
+            lab2rgb(
+                rgb2lab(img_rgb, channel_axis=channel_axis),
+                channel_axis=channel_axis
+            ),
+            img_rgb,
+        )
 
     def test_rgb2lab_dtype(self):
         img = self.colbars_array.astype('float64')
@@ -407,20 +515,30 @@ class TestColorconv(TestCase):
                                   self.luv_array, decimal=3)
 
         # Test the conversion with the rest of the illuminants.
-        for I in ["d50", "d55", "d65", "d75"]:
-            for obs in ["2", "10"]:
-                fname = "color/tests/data/luv_array_{0}_{1}.npy".format(I, obs)
+        for I in ["A", "B", "C", "d50", "d55", "d65"]:
+            I = I.lower()
+            for obs in ["2", "10", "R"]:
+                obs = obs.lower()
+                fname = f'color/tests/data/luv_array_{I}_{obs}.npy'
                 luv_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(luv_array_I_obs,
                                           xyz2luv(self.xyz_array, I, obs),
                                           decimal=2)
-        for I in ["a", "e"]:
-            fname = "color/tests/data/luv_array_{0}_2.npy".format(I)
+        for I in ["d75", "e"]:
+            fname = f'color/tests/data/luv_array_{I}_2.npy'
             luv_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(luv_array_I_obs,
                                       xyz2luv(self.xyz_array, I, "2"),
                                       decimal=2)
 
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_xyz2luv_channel_axis(self, channel_axis):
+        # test conversion with channels along a specified axis
+        xyz = np.moveaxis(self.xyz_array, source=-1, destination=channel_axis)
+        luv = xyz2luv(xyz, channel_axis=channel_axis)
+        luv = np.moveaxis(luv, source=channel_axis, destination=-1)
+        assert_array_almost_equal(luv, self.luv_array, decimal=3)
+
     def test_xyz2luv_dtype(self):
         img = self.xyz_array.astype('float64')
         img32 = img.astype('float32')
@@ -433,18 +551,28 @@ class TestColorconv(TestCase):
                                   self.xyz_array, decimal=3)
 
         # Test the conversion with the rest of the illuminants.
-        for I in ["d50", "d55", "d65", "d75"]:
-            for obs in ["2", "10"]:
-                fname = "color/tests/data/luv_array_{0}_{1}.npy".format(I, obs)
+        for I in ["A", "B", "C", "d50", "d55", "d65"]:
+            I = I.lower()
+            for obs in ["2", "10", "R"]:
+                obs = obs.lower()
+                fname = f'color/tests/data/luv_array_{I}_{obs}.npy'
                 luv_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, obs),
                                           self.xyz_array, decimal=3)
-        for I in ["a", "e"]:
-            fname = "color/tests/data/luv_array_{0}_2.npy".format(I, obs)
+        for I in ["d75", "e"]:
+            fname = f'color/tests/data/luv_array_{I}_2.npy'
             luv_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, "2"),
                                       self.xyz_array, decimal=3)
 
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_luv2xyz_channel_axis(self, channel_axis):
+        # test conversion with channels along a specified axis
+        luv = np.moveaxis(self.luv_array, source=-1, destination=channel_axis)
+        xyz = luv2xyz(luv, channel_axis=channel_axis)
+        xyz = np.moveaxis(xyz, source=channel_axis, destination=-1)
+        assert_array_almost_equal(xyz, self.xyz_array, decimal=3)
+
     def test_luv2xyz_dtype(self):
         img = self.luv_array.astype('float64')
         img32 = img.astype('float32')
@@ -486,9 +614,17 @@ class TestColorconv(TestCase):
         assert luv2rgb(img).dtype == img.dtype
         assert luv2rgb(img32).dtype == img32.dtype
 
-    def test_luv_rgb_roundtrip(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1 -2])
+    def test_luv_rgb_roundtrip(self, channel_axis):
         img_rgb = img_as_float(self.img_rgb)
-        assert_array_almost_equal(luv2rgb(rgb2luv(img_rgb)), img_rgb)
+        img_rgb = np.moveaxis(img_rgb, source=-1, destination=channel_axis)
+        assert_array_almost_equal(
+            luv2rgb(
+                rgb2luv(img_rgb, channel_axis=channel_axis),
+                channel_axis=channel_axis
+            ),
+            img_rgb,
+        )
 
     def test_lab_rgb_outlier(self):
         lab_array = np.ones((3, 1, 3))
@@ -510,10 +646,15 @@ class TestColorconv(TestCase):
             with expected_warnings(['Color data out of range']):
                 lab2xyz(lab)
 
-    def test_lab_lch_roundtrip(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_lab_lch_roundtrip(self, channel_axis):
         rgb = img_as_float(self.img_rgb)
-        lab = rgb2lab(rgb)
-        lab2 = lch2lab(lab2lch(lab))
+        rgb = np.moveaxis(rgb, source=-1, destination=channel_axis)
+        lab = rgb2lab(rgb, channel_axis=channel_axis)
+        lab2 = lch2lab(
+            lab2lch(lab, channel_axis=channel_axis),
+            channel_axis=channel_axis,
+        )
         assert_array_almost_equal(lab2, lab)
 
     def test_rgb_lch_roundtrip(self):
@@ -551,22 +692,51 @@ class TestColorconv(TestCase):
         assert_array_almost_equal(rgb2yuv(rgb), np.array([[[1, 0, 0]]]))
         assert_array_almost_equal(rgb2yiq(rgb), np.array([[[1, 0, 0]]]))
         assert_array_almost_equal(rgb2ypbpr(rgb), np.array([[[1, 0, 0]]]))
-        assert_array_almost_equal(rgb2ycbcr(rgb), np.array([[[235, 128, 128]]]))
+        assert_array_almost_equal(
+            rgb2ycbcr(rgb), np.array([[[235, 128, 128]]])
+        )
         assert_array_almost_equal(rgb2ydbdr(rgb), np.array([[[1, 0, 0]]]))
         rgb = np.array([[[0.0, 1.0, 0.0]]])
-        assert_array_almost_equal(rgb2yuv(rgb), np.array([[[0.587, -0.28886916, -0.51496512]]]))
-        assert_array_almost_equal(rgb2yiq(rgb), np.array([[[0.587, -0.27455667, -0.52273617]]]))
-        assert_array_almost_equal(rgb2ypbpr(rgb), np.array([[[0.587, -0.331264, -0.418688]]]))
-        assert_array_almost_equal(rgb2ycbcr(rgb), np.array([[[144.553,   53.797,   34.214]]]))
-        assert_array_almost_equal(rgb2ydbdr(rgb), np.array([[[0.587, -0.883, 1.116]]]))
+        assert_array_almost_equal(
+            rgb2yuv(rgb), np.array([[[0.587, -0.28886916, -0.51496512]]])
+        )
+        assert_array_almost_equal(
+            rgb2yiq(rgb), np.array([[[0.587, -0.27455667, -0.52273617]]])
+        )
+        assert_array_almost_equal(
+            rgb2ypbpr(rgb), np.array([[[0.587, -0.331264, -0.418688]]])
+        )
+        assert_array_almost_equal(
+            rgb2ycbcr(rgb), np.array([[[144.553, 53.797, 34.214]]])
+        )
+        assert_array_almost_equal(
+            rgb2ydbdr(rgb), np.array([[[0.587, -0.883, 1.116]]])
+        )
 
-    def test_yuv_roundtrip(self):
+    @pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+    def test_yuv_roundtrip(self, channel_axis):
         img_rgb = img_as_float(self.img_rgb)[::16, ::16]
-        assert_array_almost_equal(yuv2rgb(rgb2yuv(img_rgb)), img_rgb)
-        assert_array_almost_equal(yiq2rgb(rgb2yiq(img_rgb)), img_rgb)
-        assert_array_almost_equal(ypbpr2rgb(rgb2ypbpr(img_rgb)), img_rgb)
-        assert_array_almost_equal(ycbcr2rgb(rgb2ycbcr(img_rgb)), img_rgb)
-        assert_array_almost_equal(ydbdr2rgb(rgb2ydbdr(img_rgb)), img_rgb)
+        img_rgb = np.moveaxis(img_rgb, source=-1, destination=channel_axis)
+        assert_array_almost_equal(
+            yuv2rgb(rgb2yuv(img_rgb, channel_axis=channel_axis),
+                    channel_axis=channel_axis),
+            img_rgb)
+        assert_array_almost_equal(
+            yiq2rgb(rgb2yiq(img_rgb, channel_axis=channel_axis),
+                    channel_axis=channel_axis),
+            img_rgb)
+        assert_array_almost_equal(
+            ypbpr2rgb(rgb2ypbpr(img_rgb, channel_axis=channel_axis),
+                      channel_axis=channel_axis),
+            img_rgb)
+        assert_array_almost_equal(
+            ycbcr2rgb(rgb2ycbcr(img_rgb, channel_axis=channel_axis),
+                      channel_axis=channel_axis),
+            img_rgb)
+        assert_array_almost_equal(
+            ydbdr2rgb(rgb2ydbdr(img_rgb, channel_axis=channel_axis),
+                      channel_axis=channel_axis),
+            img_rgb)
 
     def test_rgb2yuv_dtype(self):
         img = self.colbars_array.astype('float64')
@@ -618,58 +788,51 @@ def test_gray2rgb():
 
 def test_gray2rgb_rgb():
     x = np.random.rand(5, 5, 4)
-    with expected_warnings(['Pass-through of possibly RGB images']):
-        y = gray2rgb(x)
-    assert_equal(x, y)
-
-
-def test_gray2rgb_alpha():
-    x = np.random.random((5, 5, 4))
-    with expected_warnings(['Pass-through of possibly RGB images']):
-        assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 4))
-    with expected_warnings(['Pass-through of possibly RGB images',
-                            'alpha argument is deprecated']):
-        assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
-    with expected_warnings(['Pass-through of possibly RGB images',
-                            'alpha argument is deprecated']):
-        assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
-
-    x = np.random.random((5, 5, 3))
-    with expected_warnings(['Pass-through of possibly RGB images']):
-        assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 3))
-    with expected_warnings(['Pass-through of possibly RGB images',
-                            'alpha argument is deprecated']):
-        assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
-    with expected_warnings(['Pass-through of possibly RGB images',
-                            'alpha argument is deprecated']):
-        assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
-
-    with expected_warnings(['alpha argument is deprecated']):
-        assert_equal(gray2rgb(np.array([[1, 2], [3, 4.]]),
-                              alpha=True)[0, 0, 3], 1)
-    with expected_warnings(['alpha argument is deprecated']):
-        assert_equal(gray2rgb(np.array([[1, 2], [3, 4]], dtype=np.uint8),
-                              alpha=True)[0, 0, 3], 255)
+    y = gray2rgb(x)
+    assert y.shape == (x.shape + (3,))
+    for i in range(3):
+        assert_equal(x, y[..., i])
 
 
 @pytest.mark.parametrize("shape", [(5, 5), (5, 5, 4), (5, 4, 5, 4)])
-def test_gray2rgba(shape):
+@pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+def test_gray2rgba(shape, channel_axis):
     # nD case
     img = np.random.random(shape)
-    rgba = gray2rgba(img)
+    rgba = gray2rgba(img, channel_axis=channel_axis)
+    assert rgba.ndim == img.ndim + 1
 
     # Shape check
-    assert_equal(rgba.shape, shape + (4, ))
+    new_axis_loc = channel_axis % rgba.ndim
+    assert_equal(rgba.shape,
+                 shape[:new_axis_loc] + (4, ) + shape[new_axis_loc:])
 
     # dtype check
     assert rgba.dtype == img.dtype
 
     # RGB channels check
     for channel in range(3):
-        assert_equal(rgba[..., channel], img)
+        assert_equal(rgba[slice_at_axis(channel, axis=new_axis_loc)], img)
 
     # Alpha channel check
-    assert_equal(rgba[..., 3], 1.0)
+    assert_equal(rgba[slice_at_axis(3, axis=new_axis_loc)], 1.0)
+
+
+@pytest.mark.parametrize("shape", [(5, 5), (5, 5, 4), (5, 4, 5, 4)])
+@pytest.mark.parametrize("channel_axis", [0, 1, -1, -2])
+def test_gray2rgb_channel_axis(shape, channel_axis):
+    # nD case
+    img = np.random.random(shape)
+    rgb = gray2rgb(img, channel_axis=channel_axis)
+    assert rgb.ndim == img.ndim + 1
+
+    # Shape check
+    new_axis_loc = channel_axis % rgb.ndim
+    assert_equal(rgb.shape,
+                 shape[:new_axis_loc] + (3, ) + shape[new_axis_loc:])
+
+    # dtype check
+    assert rgb.dtype == img.dtype
 
 
 def test_gray2rgba_dtype():
@@ -709,14 +872,13 @@ def test_gray2rgba_alpha():
 
     # Warning about alpha cast
     alpha = 0.5
-    with expected_warnings(["alpha can't be safely cast to image dtype"]):
+    with expected_warnings(["alpha cannot be safely cast to image dtype"]):
         rgba = gray2rgba(img_u8, alpha)
         assert_equal(rgba[..., :3], gray2rgb(img_u8))
 
     # Invalid shape
     alpha = np.random.random((5, 5, 1))
-    expected_err_msg = ("could not broadcast input array from shape (5,5,1) "
-                        "into shape (5,5)")
+    expected_err_msg = ("alpha.shape must match image.shape")
 
     with pytest.raises(ValueError) as err:
         rgba = gray2rgba(img, alpha)
@@ -728,18 +890,8 @@ def test_gray2rgba_alpha():
                                     (4, 5, 4, 5, 3)]))
 def test_nD_gray_conversion(func, shape):
     img = np.random.rand(*shape)
-
-    msg_list = []
-    if img.ndim == 3 and func == gray2rgb:
-        msg_list.append('Pass-through of possibly RGB images in gray2rgb')
-    elif img.ndim == 2 and func == rgb2gray:
-        msg_list.append('The behavior of rgb2gray will change')
-
-    with expected_warnings(msg_list):
-        out = func(img)
-
+    out = func(img)
     common_ndim = min(out.ndim, len(shape))
-
     assert out.shape[:common_ndim] == shape[:common_ndim]
 
 
@@ -775,3 +927,44 @@ def test_rgba2rgb_nD(shape):
     expected_shape = shape[:-1] + (3, )
 
     assert out.shape == expected_shape
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_rgba2rgb_dtypes(dtype):
+    rgba = np.array([[[0, 0.5, 1, 0],
+                      [0, 0.5, 1, 1],
+                      [0, 0.5, 1, 0.5]]]).astype(dtype=dtype)
+    rgb = rgba2rgb(rgba)
+    float_dtype = _supported_float_type(rgba.dtype)
+    assert rgb.dtype == float_dtype
+    expected = np.array([[[1, 1, 1],
+                          [0, 0.5, 1],
+                          [0.5, 0.75, 1]]]).astype(float)
+    assert rgb.shape == expected.shape
+    assert_almost_equal(rgb, expected)
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_lab_lch_roundtrip_dtypes(dtype):
+    rgb = img_as_float(data.colorwheel()).astype(dtype=dtype, copy=False)
+    lab = rgb2lab(rgb)
+    float_dtype = _supported_float_type(dtype)
+    assert lab.dtype == float_dtype
+    lab2 = lch2lab(lab2lch(lab))
+    decimal = 4 if float_dtype == np.float32 else 7
+    assert_array_almost_equal(lab2, lab, decimal=decimal)
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_rgb2hsv_dtypes(dtype):
+    rgb = img_as_float(data.colorwheel())[::16, ::16]
+    rgb = rgb.astype(dtype=dtype, copy=False)
+    hsv = rgb2hsv(rgb).reshape(-1, 3)
+    float_dtype = _supported_float_type(dtype)
+    assert hsv.dtype == float_dtype
+    # ground truth from colorsys
+    gt = np.array([colorsys.rgb_to_hsv(pt[0], pt[1], pt[2])
+                   for pt in rgb.reshape(-1, 3)]
+                  )
+    decimal = 3 if float_dtype == np.float32 else 7
+    assert_array_almost_equal(hsv, gt, decimal=decimal)
