@@ -1,8 +1,7 @@
 import numpy as np
 
-from ..color.colorconv import rgb2gray, rgba2rgb
 from ..util.dtype import dtype_range, dtype_limits
-from .._shared.utils import warn
+from .._shared import utils
 
 
 __all__ = ['histogram', 'cumulative_distribution', 'equalize_hist',
@@ -30,12 +29,25 @@ def _offset_array(arr, low_boundary, high_boundary):
             # prevent overflow errors when offsetting
             arr = arr.astype(offset_dtype)
         arr = arr - offset
-    else:
-        offset = 0
-    return arr, offset
+    return arr
 
 
-def _bincount_histogram(image, source_range):
+def _bincount_histogram_centers(image, source_range):
+    """Compute bin centers for bincount-based histogram."""
+    if source_range not in ['image', 'dtype']:
+        raise ValueError(
+            f'Incorrect value for `source_range` argument: {source_range}'
+        )
+    if source_range == 'image':
+        image_min = int(image.min().astype(np.int64))
+        image_max = int(image.max().astype(np.int64))
+    elif source_range == 'dtype':
+        image_min, image_max = dtype_limits(image, clip_negative=False)
+    bin_centers = np.arange(image_min, image_max + 1)
+    return bin_centers
+
+
+def _bincount_histogram(image, source_range, bin_centers=None):
     """
     Efficient histogram calculation for an image of integers.
 
@@ -58,32 +70,132 @@ def _bincount_histogram(image, source_range):
     bin_centers : array
         The values at the center of the bins.
     """
-    if source_range not in ['image', 'dtype']:
-        raise ValueError('Incorrect value for `source_range` argument: {}'.format(source_range))
-    if source_range == 'image':
-        image_min = int(image.min().astype(np.int64))
-        image_max = int(image.max().astype(np.int64))
-    elif source_range == 'dtype':
-        image_min, image_max = dtype_limits(image, clip_negative=False)
-    image, offset = _offset_array(image, image_min, image_max)
-    hist = np.bincount(image.ravel(), minlength=image_max - image_min + 1)
-    bin_centers = np.arange(image_min, image_max + 1)
+    if bin_centers is None:
+        bin_centers = _bincount_histogram_centers(image, source_range)
+    image_min, image_max = bin_centers[0], bin_centers[-1]
+    image = _offset_array(image, image_min, image_max)
+    hist = np.bincount(
+        image.ravel(), minlength=image_max - min(image_min, 0) + 1
+    )
     if source_range == 'image':
         idx = max(image_min, 0)
         hist = hist[idx:]
     return hist, bin_centers
 
 
-def histogram(image, nbins=256, source_range='image', normalize=False):
+def _get_outer_edges(image, hist_range):
+    """Determine the outer bin edges to use for `numpy.histogram`.
+
+    These are obtained from either the image or hist_range.
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    hist_range: 2-tuple of int or None
+        Range of values covered by the histogram bins. If None, the minimum
+        and maximum values of `image` are used.
+
+    Returns
+    -------
+    first_edge, last_edge : int
+        The range spanned by the histogram bins.
+
+    Notes
+    -----
+    This function is adapted from ``np.lib.histograms._get_outer_edges``.
+    """
+    if hist_range is not None:
+        first_edge, last_edge = hist_range
+        if first_edge > last_edge:
+            raise ValueError(
+                "max must be larger than min in hist_range parameter."
+            )
+        if not (np.isfinite(first_edge) and np.isfinite(last_edge)):
+            raise ValueError(
+                f'supplied hist_range of [{first_edge}, {last_edge}] is '
+                f'not finite'
+            )
+    elif image.size == 0:
+        # handle empty arrays. Can't determine hist_range, so use 0-1.
+        first_edge, last_edge = 0, 1
+    else:
+        first_edge, last_edge = image.min(), image.max()
+        if not (np.isfinite(first_edge) and np.isfinite(last_edge)):
+            raise ValueError(
+                f'autodetected hist_range of [{first_edge}, {last_edge}] is '
+                f'not finite'
+            )
+
+    # expand empty hist_range to avoid divide by zero
+    if first_edge == last_edge:
+        first_edge = first_edge - 0.5
+        last_edge = last_edge + 0.5
+
+    return first_edge, last_edge
+
+
+def _get_bin_edges(image, nbins, hist_range):
+    """Computes histogram bins for use with `numpy.histogram`.
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    nbins : int
+        The number of bins.
+    hist_range: 2-tuple of int
+        Range of values covered by the histogram bins.
+
+    Returns
+    -------
+    bin_edges : ndarray
+        The histogram bin edges.
+
+    Notes
+    -----
+    This function is a simplified version of
+    ``np.lib.histograms._get_bin_edges`` that only supports uniform bins.
+    """
+    first_edge, last_edge = _get_outer_edges(image, hist_range)
+    # numpy/gh-10322 means that type resolution rules are dependent on array
+    # shapes. To avoid this causing problems, we pick a type now and stick
+    # with it throughout.
+    bin_type = np.result_type(first_edge, last_edge, image)
+    if np.issubdtype(bin_type, np.integer):
+        bin_type = np.result_type(bin_type, float)
+
+    # compute bin edges
+    bin_edges = np.linspace(
+        first_edge, last_edge, nbins + 1, endpoint=True, dtype=bin_type
+    )
+    return bin_edges
+
+
+def _get_numpy_hist_range(image, source_range):
+    if source_range == 'image':
+        hist_range = None
+    elif source_range == 'dtype':
+        hist_range = dtype_limits(image, clip_negative=False)
+    else:
+        ValueError('Wrong value for the `source_range` argument')
+    return hist_range
+
+
+@utils.channel_as_last_axis(multichannel_output=False)
+def histogram(image, nbins=256, source_range='image', normalize=False, *,
+              channel_axis=None):
     """Return histogram of image.
 
     Unlike `numpy.histogram`, this function returns the centers of bins and
     does not rebin integer arrays. For integer arrays, each integer value has
     its own bin, which improves speed and intensity-resolution.
 
-    The histogram is computed on the flattened image: for color images, the
-    function should be used separately on each channel to obtain a histogram
-    for each color channel.
+    If `channel_axis` is not set, the histogram is computed on the flattened
+    image. For color or multichannel images, set ``channel_axis`` to use a
+    common binning for all channels. Alternatively, one may apply the function
+    separately on each channel to obtain a histogram for each color channel
+    with separate binning.
 
     Parameters
     ----------
@@ -98,11 +210,16 @@ def histogram(image, nbins=256, source_range='image', normalize=False):
         of that data type.
     normalize : bool, optional
         If True, normalize the histogram by the sum of its values.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
 
     Returns
     -------
     hist : array
-        The values of the histogram.
+        The values of the histogram. When ``channel_axis`` is not None, hist
+        will be a 2D array where the first axis corresponds to channels.
     bin_centers : array
         The values at the center of the bins.
 
@@ -120,23 +237,67 @@ def histogram(image, nbins=256, source_range='image', normalize=False):
     (array([ 93585, 168559]), array([0.25, 0.75]))
     """
     sh = image.shape
-    if len(sh) == 3 and sh[-1] < 4:
-        warn("This might be a color image. The histogram will be "
-             "computed on the flattened image. You can instead "
-             "apply this function to each color channel.")
+    if len(sh) == 3 and sh[-1] < 4 and channel_axis is None:
+        utils.warn('This might be a color image. The histogram will be '
+                   'computed on the flattened image. You can instead '
+                   'apply this function to each color channel, or set '
+                   'channel_axis.')
+
+    if channel_axis is not None:
+        channels = sh[-1]
+        hist = []
+
+        # compute bins based on the raveled array
+        if np.issubdtype(image.dtype, np.integer):
+            # here bins corresponds to the bin centers
+            bins = _bincount_histogram_centers(image, source_range)
+        else:
+            # determine the bin edges for np.histogram
+            hist_range = _get_numpy_hist_range(image, source_range)
+            bins = _get_bin_edges(image, nbins, hist_range)
+
+        for chan in range(channels):
+            h, bc = _histogram(image[..., chan], bins, source_range, normalize)
+            hist.append(h)
+        # Convert to numpy arrays
+        bin_centers = np.asarray(bc)
+        hist = np.stack(hist, axis=0)
+    else:
+        hist, bin_centers = _histogram(image, nbins, source_range, normalize)
+
+    return hist, bin_centers
+
+
+def _histogram(image, bins, source_range, normalize):
+    """
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    bins : int or ndarray
+        The number of histogram bins. For images with integer dtype, an array
+        containing the bin centers can also be provided. For images with
+        floating point dtype, this can be an array of bin_edges for use by
+        ``np.histogram``.
+    source_range : string, optional
+        'image' (default) determines the range from the input image.
+        'dtype' determines the range from the expected range of the images
+        of that data type.
+    normalize : bool, optional
+        If True, normalize the histogram by the sum of its values.
+    """
 
     image = image.flatten()
     # For integer types, histogramming with bincount is more efficient.
     if np.issubdtype(image.dtype, np.integer):
-        hist, bin_centers = _bincount_histogram(image, source_range)
+        bin_centers = bins if isinstance(bins, np.ndarray) else None
+        hist, bin_centers = _bincount_histogram(
+            image, source_range, bin_centers
+        )
     else:
-        if source_range == 'image':
-            hist_range = None
-        elif source_range == 'dtype':
-            hist_range = dtype_limits(image, clip_negative=False)
-        else:
-            ValueError('Wrong value for the `source_range` argument')
-        hist, bin_edges = np.histogram(image, bins=nbins, range=hist_range)
+        hist_range = _get_numpy_hist_range(image, source_range)
+        hist, bin_edges = np.histogram(image, bins=bins, range=hist_range)
         bin_centers = (bin_edges[:-1] + bin_edges[1:]) / 2.
 
     if normalize:
@@ -181,6 +342,11 @@ def cumulative_distribution(image, nbins=256):
     hist, bin_centers = histogram(image, nbins)
     img_cdf = hist.cumsum()
     img_cdf = img_cdf / float(img_cdf[-1])
+
+    # cast img_cdf to single precision for float32 or float16 inputs
+    cdf_dtype = utils._supported_float_type(image.dtype)
+    img_cdf = img_cdf.astype(cdf_dtype, copy=False)
+
     return img_cdf, bin_centers
 
 
@@ -195,7 +361,7 @@ def equalize_hist(image, nbins=256, mask=None):
         Number of bins for image histogram. Note: this argument is
         ignored for integer images, for which each integer is its own
         bin.
-    mask: ndarray of bools or 0s and 1s, optional
+    mask : ndarray of bools or 0s and 1s, optional
         Array of same shape as `image`. Only points at which mask == True
         are used for the equalization, which is applied to the whole image.
 
@@ -220,7 +386,10 @@ def equalize_hist(image, nbins=256, mask=None):
     else:
         cdf, bin_centers = cumulative_distribution(image, nbins)
     out = np.interp(image.flat, bin_centers, cdf)
-    return out.reshape(image.shape)
+    out = out.reshape(image.shape)
+    # Unfortunately, np.interp currently always promotes to float64, so we
+    # have to cast back to single precision when float32 output is desired
+    return out.astype(utils._supported_float_type(image.dtype), copy=False)
 
 
 def intensity_range(image, range_values='image', clip_negative=False):
@@ -266,7 +435,7 @@ def intensity_range(image, range_values='image', clip_negative=False):
     return i_min, i_max
 
 
-def _output_dtype(dtype_or_range):
+def _output_dtype(dtype_or_range, image_dtype):
     """Determine the output dtype for rescale_intensity.
 
     The dtype is determined according to the following rules:
@@ -276,13 +445,16 @@ def _output_dtype(dtype_or_range):
       in which case the data type that can contain it will be used
       (e.g. uint16 in this case).
     - if ``dtype_or_range`` is a pair of values, the output data type will be
-      float.
+      ``_supported_float_type(image_dtype)``. This preserves float32 output for
+      float32 inputs.
 
     Parameters
     ----------
     dtype_or_range : type, string, or 2-tuple of int/float
         The desired range for the output, expressed as either a NumPy dtype or
         as a (min, max) pair of numbers.
+    image_dtype : np.dtype
+        The input image dtype.
 
     Returns
     -------
@@ -291,7 +463,7 @@ def _output_dtype(dtype_or_range):
     """
     if type(dtype_or_range) in [list, tuple, np.ndarray]:
         # pair of values: always return float.
-        return float
+        return utils._supported_float_type(image_dtype)
     if type(dtype_or_range) == type:
         # already a type: return it
         return dtype_or_range
@@ -344,7 +516,7 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
     Notes
     -----
     .. versionchanged:: 0.17
-        The dtype of the output array has changed to match the output dtype, or
+        The dtype of the output array has changed to match the input dtype, or
         float if the output range is specified by a pair of floats.
 
     See Also
@@ -403,16 +575,16 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
     array([127, 127, 127], dtype=int32)
     """
     if out_range in ['dtype', 'image']:
-        out_dtype = _output_dtype(image.dtype.type)
+        out_dtype = _output_dtype(image.dtype.type, image.dtype)
     else:
-        out_dtype = _output_dtype(out_range)
+        out_dtype = _output_dtype(out_range, image.dtype)
 
     imin, imax = map(float, intensity_range(image, in_range))
     omin, omax = map(float, intensity_range(image, out_range,
                                             clip_negative=(imin >= 0)))
 
     if np.any(np.isnan([imin, imax, omin, omax])):
-        warn(
+        utils.warn(
             "One or more intensity levels are NaN. Rescaling will broadcast "
             "NaN to the full image. Provide intensity levels yourself to "
             "avoid this. E.g. with np.nanmin(image), np.nanmax(image).",
@@ -437,10 +609,11 @@ def _assert_non_negative(image):
 
 
 def _adjust_gamma_u8(image, gamma, gain):
-    """LUT based implmentation of gamma adjustement.
+    """LUT based implementation of gamma adjustment.
 
     """
-    lut = (255 * gain * (np.linspace(0, 1, 256) ** gamma)).astype('uint8')
+    lut = (255 * gain * (np.linspace(0, 1, 256) ** gamma))
+    lut = np.minimum(lut, 255).astype('uint8')
     return lut[image]
 
 
@@ -512,8 +685,9 @@ def adjust_log(image, gain=1, inv=False):
     """Performs Logarithmic correction on the input image.
 
     This function transforms the input image pixelwise according to the
-    equation ``O = gain*log(1 + I)`` after scaling each pixel to the range 0 to 1.
-    For inverse logarithmic correction, the equation is ``O = gain*(2**I - 1)``.
+    equation ``O = gain*log(1 + I)`` after scaling each pixel to the range
+    0 to 1. For inverse logarithmic correction, the equation is
+    ``O = gain*(2**I - 1)``.
 
     Parameters
     ----------
@@ -585,7 +759,7 @@ def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
     ----------
     .. [1] Gustav J. Braun, "Image Lightness Rescaling Using Sigmoidal Contrast
            Enhancement Functions",
-           http://www.cis.rit.edu/fairchild/PDFs/PAP07.pdf
+           http://markfairchild.org/PDFs/PAP07.pdf
 
     """
     _assert_non_negative(image)
@@ -625,6 +799,11 @@ def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
     out : bool
         True when the image is determined to be low contrast.
 
+    Notes
+    -----
+    For boolean images, this function returns False only if all values are
+    the same (the method, threshold, and percentile arguments are ignored).
+
     References
     ----------
     .. [1] https://scikit-image.org/docs/dev/user_guide/data_types.html
@@ -641,7 +820,13 @@ def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
     False
     """
     image = np.asanyarray(image)
+
+    if image.dtype == bool:
+        return not ((image.max() == 1) and (image.min() == 0))
+
     if image.ndim == 3:
+        from ..color import rgb2gray, rgba2rgb  # avoid circular import
+
         if image.shape[2] == 4:
             image = rgba2rgb(image)
         if image.shape[2] == 3:
