@@ -1,19 +1,18 @@
 from itertools import combinations_with_replacement
+from warnings import warn
 
 import numpy as np
 from scipy import ndimage as ndi
-from scipy import stats
-from scipy import spatial
+from scipy import spatial, stats
 
+from .._shared.filters import gaussian
+from .._shared.utils import _supported_float_type, safe_as_int
+from ..transform import integral_image
 from ..util import img_as_float
+from ._hessian_det_appx import _hessian_matrix_det
+from .corner_cy import _corner_fast, _corner_moravec, _corner_orientations
 from .peak import peak_local_max
 from .util import _prepare_grayscale_input_2D, _prepare_grayscale_input_nD
-from .corner_cy import _corner_fast
-from ._hessian_det_appx import _hessian_matrix_det
-from ..transform import integral_image
-from .._shared.utils import safe_as_int
-from .corner_cy import _corner_moravec, _corner_orientations
-from warnings import warn
 
 
 def _compute_derivatives(image, mode='constant', cval=0):
@@ -58,9 +57,12 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0, order=None):
     ----------
     image : ndarray
         Input image.
-    sigma : float, optional
+    sigma : float or array-like of float, optional
         Standard deviation used for the Gaussian kernel, which is used as a
         weighting function for the local summation of squared differences.
+        If sigma is an iterable, its length must be equal to `image.ndim` and
+        each element is used for the Gaussian kernel applied along its
+        respective axis.
     mode : {'constant', 'reflect', 'wrap', 'nearest', 'mirror'}, optional
         How to handle values outside the image borders.
     cval : float, optional
@@ -116,6 +118,12 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0, order=None):
         else:
             order = 'rc'
 
+    if not np.isscalar(sigma):
+        sigma = tuple(sigma)
+        if len(sigma) != image.ndim:
+            raise ValueError('sigma must have as many elements as image '
+                             'has axes')
+
     image = _prepare_grayscale_input_nD(image)
 
     derivatives = _compute_derivatives(image, mode=mode, cval=cval)
@@ -124,16 +132,16 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0, order=None):
         derivatives = reversed(derivatives)
 
     # structure tensor
-    A_elems = [ndi.gaussian_filter(der0 * der1, sigma, mode=mode, cval=cval)
+    A_elems = [gaussian(der0 * der1, sigma, mode=mode, cval=cval)
                for der0, der1 in combinations_with_replacement(derivatives, 2)]
 
     return A_elems
 
 
 def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
-    """Compute Hessian matrix.
+    """Compute the Hessian matrix.
 
-    The Hessian matrix is defined as::
+    In 2D, the Hessian matrix is defined as::
 
         H = [Hrr Hrc]
             [Hrc Hcc]
@@ -141,6 +149,8 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
     which is computed by convolving the image with the second derivatives
     of the Gaussian kernel in the respective r- and c-directions.
 
+    The implementation here also supports n-dimensional data.
+
     Parameters
     ----------
     image : ndarray
@@ -161,12 +171,10 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
 
     Returns
     -------
-    Hrr : ndarray
-        Element of the Hessian matrix for each pixel in the input image.
-    Hrc : ndarray
-        Element of the Hessian matrix for each pixel in the input image.
-    Hcc : ndarray
-        Element of the Hessian matrix for each pixel in the input image.
+    H_elems : list of ndarray
+        Upper-diagonal elements of the hessian matrix for each pixel in the
+        input image. In 2D, this will be a three element list containing [Hrr,
+        Hrc, Hcc]. In nD, the list will contain ``(n**2 + n) / 2`` arrays.
 
     Examples
     --------
@@ -183,9 +191,10 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
     """
 
     image = img_as_float(image)
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
 
-    gaussian_filtered = ndi.gaussian_filter(image, sigma=sigma,
-                                            mode=mode, cval=cval)
+    gaussian_filtered = gaussian(image, sigma=sigma, mode=mode, cval=cval)
 
     gradients = np.gradient(gaussian_filtered)
     axes = range(image.ndim)
@@ -195,7 +204,6 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
 
     H_elems = [np.gradient(gradients[ax0], axis=ax1)
                for ax0, ax1 in combinations_with_replacement(axes, 2)]
-
     return H_elems
 
 
@@ -203,14 +211,14 @@ def hessian_matrix_det(image, sigma=1, approximate=True):
     """Compute the approximate Hessian Determinant over an image.
 
     The 2D approximate method uses box filters over integral images to
-    compute the approximate Hessian Determinant, as described in [1]_.
+    compute the approximate Hessian Determinant.
 
     Parameters
     ----------
-    image : array
-        The image over which to compute Hessian Determinant.
+    image : ndarray
+        The image over which to compute the Hessian Determinant.
     sigma : float, optional
-        Standard deviation used for the Gaussian kernel, used for the Hessian
+        Standard deviation of the Gaussian kernel used for the Hessian
         matrix.
     approximate : bool, optional
         If ``True`` and the image is 2D, use a much faster approximate
@@ -236,6 +244,8 @@ def hessian_matrix_det(image, sigma=1, approximate=True):
     computed the Hessian and took its determinant.
     """
     image = img_as_float(image)
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
     if image.ndim == 2 and approximate:
         integral = integral_image(image)
         return np.array(_hessian_matrix_det(integral, sigma))
@@ -295,7 +305,8 @@ def _symmetric_image(S_elems):
         containing the matrix corresponding to each coordinate.
     """
     image = S_elems[0]
-    symmetric_image = np.zeros(image.shape + (image.ndim, image.ndim))
+    symmetric_image = np.zeros(image.shape + (image.ndim, image.ndim),
+                               dtype=S_elems[0].dtype)
     for idx, (row, col) in \
             enumerate(combinations_with_replacement(range(image.ndim), 2)):
         symmetric_image[..., row, col] = S_elems[idx]
@@ -365,7 +376,7 @@ def structure_tensor_eigvals(Axx, Axy, Ayy):
     >>> square = np.zeros((5, 5))
     >>> square[2, 2] = 1
     >>> Arr, Arc, Acc = structure_tensor(square, sigma=0.1, order='rc')
-    >>> structure_tensor_eigvals(Acc, Arc, Arr)[0]
+    >>> structure_tensor_eigvals(Acc, Arc, Arr)[0]  # doctest: +SKIP
     array([[0., 0., 0., 0., 0.],
            [0., 2., 4., 2., 0.],
            [0., 4., 0., 4., 0.],
@@ -420,7 +431,7 @@ def shape_index(image, sigma=1, mode='constant', cval=0):
     single valued measure of local curvature, assuming the image as a 3D plane
     with intensities representing heights.
 
-    It is derived from the eigen values of the Hessian, and its
+    It is derived from the eigenvalues of the Hessian, and its
     value ranges from -1 to 1 (and is undefined (=NaN) in *flat* regions),
     with following ranges representing following shapes:
 
@@ -442,7 +453,7 @@ def shape_index(image, sigma=1, mode='constant', cval=0):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     sigma : float, optional
         Standard deviation used for the Gaussian kernel, which is used for
@@ -482,7 +493,9 @@ def shape_index(image, sigma=1, mode='constant', cval=0):
     H = hessian_matrix(image, sigma=sigma, mode=mode, cval=cval, order='rc')
     l1, l2 = hessian_matrix_eigvals(H)
 
-    return (2.0 / np.pi) * np.arctan((l2 + l1) / (l2 - l1))
+    # don't warn on divide by 0 as occurs in the docstring example
+    with np.errstate(divide='ignore', invalid='ignore'):
+        return (2.0 / np.pi) * np.arctan((l2 + l1) / (l2 - l1))
 
 
 def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
@@ -498,7 +511,7 @@ def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     mode : {'constant', 'reflect', 'wrap', 'nearest', 'mirror'}, optional
         How to handle values outside the image borders.
@@ -518,6 +531,9 @@ def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
            :DOI:`10.1016/0167-8655(82)90020-4`
     """
 
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
+
     imy, imx = _compute_derivatives(image, mode=mode, cval=cval)
     imxy, imxx = _compute_derivatives(imx, mode=mode, cval=cval)
     imyy, imyx = _compute_derivatives(imy, mode=mode, cval=cval)
@@ -525,7 +541,7 @@ def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
     numerator = (imxx * imy ** 2 + imyy * imx ** 2 - 2 * imxy * imx * imy)
     denominator = (imx ** 2 + imy ** 2)
 
-    response = np.zeros_like(image, dtype=np.double)
+    response = np.zeros_like(image, dtype=float_dtype)
 
     mask = denominator != 0
     response[mask] = numerator[mask] / denominator[mask]
@@ -552,7 +568,7 @@ def corner_harris(image, method='k', k=0.05, eps=1e-6, sigma=1):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     method : {'k', 'eps'}, optional
         Method to compute the response image from the auto-correlation matrix.
@@ -628,7 +644,7 @@ def corner_shi_tomasi(image, sigma=1):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     sigma : float, optional
         Standard deviation used for the Gaussian kernel, which is used as
@@ -691,7 +707,7 @@ def corner_foerstner(image, sigma=1):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     sigma : float, optional
         Standard deviation used for the Gaussian kernel, which is used as
@@ -748,8 +764,8 @@ def corner_foerstner(image, sigma=1):
     # trace
     traceA = Arr + Acc
 
-    w = np.zeros_like(image, dtype=np.double)
-    q = np.zeros_like(image, dtype=np.double)
+    w = np.zeros_like(image, dtype=detA.dtype)
+    q = np.zeros_like(w)
 
     mask = traceA != 0
 
@@ -764,7 +780,7 @@ def corner_fast(image, n=12, threshold=0.15):
 
     Parameters
     ----------
-    image : 2D ndarray
+    image : (M, N) ndarray
         Input image.
     n : int, optional
         Minimum number of consecutive pixels out of 16 pixels on the circle
@@ -836,9 +852,9 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
-    corners : (N, 2) ndarray
+    corners : (K, 2) ndarray
         Corner coordinates `(row, col)`.
     window_size : int, optional
         Search window size for subpixel estimation.
@@ -847,7 +863,7 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
 
     Returns
     -------
-    positions : (N, 2) ndarray
+    positions : (K, 2) ndarray
         Subpixel corner positions. NaN for "not classified" corners.
 
     References
@@ -886,16 +902,18 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
     # window extent in one direction
     wext = (window_size - 1) // 2
 
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
     image = np.pad(image, pad_width=wext, mode='constant', constant_values=0)
 
     # add pad width, make sure to not modify the input values in-place
     corners = safe_as_int(corners + wext)
 
     # normal equation arrays
-    N_dot = np.zeros((2, 2), dtype=np.double)
-    N_edge = np.zeros((2, 2), dtype=np.double)
-    b_dot = np.zeros((2, ), dtype=np.double)
-    b_edge = np.zeros((2, ), dtype=np.double)
+    N_dot = np.zeros((2, 2), dtype=float_dtype)
+    N_edge = np.zeros((2, 2), dtype=float_dtype)
+    b_dot = np.zeros((2, ), dtype=float_dtype)
+    b_edge = np.zeros((2, ), dtype=float_dtype)
 
     # critical statistical test values
     redundancy = window_size ** 2 - 2
@@ -905,7 +923,7 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
     # coordinates of pixels within window
     y, x = np.mgrid[- wext:wext + 1, - wext:wext + 1]
 
-    corners_subpix = np.zeros_like(corners, dtype=np.double)
+    corners_subpix = np.zeros_like(corners, dtype=float_dtype)
 
     for i, (y0, x0) in enumerate(corners):
 
@@ -1013,7 +1031,7 @@ def corner_peaks(image, min_distance=1, threshold_abs=None, threshold_rel=None,
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     min_distance : int, optional
         The minimal allowed distance separating peaks.
@@ -1113,7 +1131,7 @@ def corner_moravec(image, window_size=1):
 
     Parameters
     ----------
-    image : ndarray
+    image : (M, N) ndarray
         Input image.
     window_size : int, optional
         Window size.
@@ -1149,7 +1167,10 @@ def corner_moravec(image, window_size=1):
            [0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0]])
     """
-    return _corner_moravec(image, window_size)
+    image = img_as_float(image)
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
+    return _corner_moravec(np.ascontiguousarray(image), window_size)
 
 
 def corner_orientations(image, corners, mask):
@@ -1163,9 +1184,9 @@ def corner_orientations(image, corners, mask):
 
     Parameters
     ----------
-    image : 2D array
+    image : (M, N) array
         Input grayscale image.
-    corners : (N, 2) array
+    corners : (K, 2) array
         Corner coordinates as ``(row, col)``.
     mask : 2D array
         Mask defining the local neighborhood of the corner used for the
@@ -1173,7 +1194,7 @@ def corner_orientations(image, corners, mask):
 
     Returns
     -------
-    orientations : (N, 1) array
+    orientations : (K, 1) array
         Orientations of corners in the range [-pi, pi].
 
     References
