@@ -1,17 +1,23 @@
-import pytest
+import math
 
 import numpy as np
-from skimage.draw import disk
-from skimage.draw.draw3d import ellipsoid
-from skimage.feature import blob_dog, blob_log, blob_doh
-from skimage.feature.blob import _blob_overlap
-import math
+import pytest
 from numpy.testing import assert_almost_equal
 
+from skimage import feature
+from skimage.draw import disk
+from skimage.draw.draw3d import ellipsoid
+from skimage.feature import blob_dog, blob_doh, blob_log
+from skimage.feature.blob import _blob_overlap
 
-def test_blob_dog():
+
+@pytest.mark.parametrize(
+    'dtype', [np.uint8, np.float16, np.float32, np.float64]
+)
+@pytest.mark.parametrize('threshold_type', ['absolute', 'relative'])
+def test_blob_dog(dtype, threshold_type):
     r2 = math.sqrt(2)
-    img = np.ones((512, 512))
+    img = np.ones((512, 512), dtype=dtype)
 
     xs, ys = disk((400, 130), 5)
     img[xs, ys] = 255
@@ -22,58 +28,108 @@ def test_blob_dog():
     xs, ys = disk((200, 350), 45)
     img[xs, ys] = 255
 
-    blobs = blob_dog(img, min_sigma=5, max_sigma=50)
+    if threshold_type == 'absolute':
+        threshold = 2.0
+        if img.dtype.kind != 'f':
+            # account for internal scaling to [0, 1] by img_as_float
+            threshold /= img.ptp()
+        threshold_rel = None
+    elif threshold_type == 'relative':
+        threshold = None
+        threshold_rel = 0.5
+
+    blobs = blob_dog(
+        img,
+        min_sigma=4,
+        max_sigma=50,
+        threshold=threshold,
+        threshold_rel=threshold_rel,
+    )
     radius = lambda x: r2 * x[2]
     s = sorted(blobs, key=radius)
     thresh = 5
+    ratio_thresh = 0.25
 
     b = s[0]
     assert abs(b[0] - 400) <= thresh
     assert abs(b[1] - 130) <= thresh
-    assert abs(radius(b) - 5) <= thresh
+    assert abs(radius(b) - 5) <= ratio_thresh * 5
 
     b = s[1]
     assert abs(b[0] - 100) <= thresh
     assert abs(b[1] - 300) <= thresh
-    assert abs(radius(b) - 25) <= thresh
+    assert abs(radius(b) - 25) <= ratio_thresh * 25
 
     b = s[2]
     assert abs(b[0] - 200) <= thresh
     assert abs(b[1] - 350) <= thresh
-    assert abs(radius(b) - 45) <= thresh
+    assert abs(radius(b) - 45) <= ratio_thresh * 45
 
     # Testing no peaks
-    img_empty = np.zeros((100,100))
+    img_empty = np.zeros((100, 100), dtype=dtype)
     assert blob_dog(img_empty).size == 0
 
+
+@pytest.mark.parametrize(
+    'dtype', [np.uint8, np.float16, np.float32, np.float64]
+)
+@pytest.mark.parametrize('threshold_type', ['absolute', 'relative'])
+def test_blob_dog_3d(dtype, threshold_type):
     # Testing 3D
     r = 10
     pad = 10
     im3 = ellipsoid(r, r, r)
     im3 = np.pad(im3, pad, mode='constant')
 
-    blobs = blob_dog(im3, min_sigma=3, max_sigma=10,
-                          sigma_ratio=1.2, threshold=0.1)
+    if threshold_type == 'absolute':
+        threshold = 0.001
+        threshold_rel = 0
+    elif threshold_type == 'relative':
+        threshold = 0
+        threshold_rel = 0.5
+
+    blobs = blob_dog(
+        im3,
+        min_sigma=3,
+        max_sigma=10,
+        sigma_ratio=1.2,
+        threshold=threshold,
+        threshold_rel=threshold_rel,
+    )
     b = blobs[0]
 
     assert b.shape == (4,)
     assert b[0] == r + pad + 1
     assert b[1] == r + pad + 1
     assert b[2] == r + pad + 1
-    assert abs(math.sqrt(3) * b[3] - r) < 1
+    assert abs(math.sqrt(3) * b[3] - r) < 1.1
 
+
+@pytest.mark.parametrize(
+    'dtype', [np.uint8, np.float16, np.float32, np.float64]
+)
+@pytest.mark.parametrize('threshold_type', ['absolute', 'relative'])
+def test_blob_dog_3d_anisotropic(dtype, threshold_type):
     # Testing 3D anisotropic
     r = 10
     pad = 10
     im3 = ellipsoid(r / 2, r, r)
     im3 = np.pad(im3, pad, mode='constant')
 
+    if threshold_type == 'absolute':
+        threshold = 0.001
+        threshold_rel = None
+    elif threshold_type == 'relative':
+        threshold = None
+        threshold_rel = 0.5
+
     blobs = blob_dog(
-        im3,
+        im3.astype(dtype, copy=False),
         min_sigma=[1.5, 3, 3],
         max_sigma=[5, 10, 10],
         sigma_ratio=1.2,
-        threshold=0.1
+        threshold=threshold,
+        threshold_rel=threshold_rel,
     )
     b = blobs[0]
 
@@ -81,19 +137,15 @@ def test_blob_dog():
     assert b[0] == r / 2 + pad + 1
     assert b[1] == r + pad + 1
     assert b[2] == r + pad + 1
-    assert abs(math.sqrt(3) * b[3] - r / 2) < 1
-    assert abs(math.sqrt(3) * b[4] - r) < 1
-    assert abs(math.sqrt(3) * b[5] - r) < 1
-
-    # Testing exclude border
-
-    # image where blob is 5 px from borders, radius 5
-    img = np.ones((512, 512))
-    xs, ys = disk((5, 5), 5)
-    img[xs, ys] = 255
+    assert abs(math.sqrt(3) * b[3] - r / 2) < 1.1
+    assert abs(math.sqrt(3) * b[4] - r) < 1.1
+    assert abs(math.sqrt(3) * b[5] - r) < 1.1
 
 
 def test_blob_dog_excl_border():
+    # Testing exclude border
+
+    # image where blob is 5 px from borders, radius 5
     img = np.ones((512, 512))
     xs, ys = disk((5, 5), 5)
     img[xs, ys] = 255
@@ -118,9 +170,30 @@ def test_blob_dog_excl_border():
     assert blobs.shape[0] == 0, msg
 
 
-def test_blob_log():
+@pytest.mark.parametrize('anisotropic', [False, True])
+@pytest.mark.parametrize('ndim', [1, 2, 3, 4])
+@pytest.mark.parametrize('function_name', ['blob_dog', 'blob_log'])
+def test_nd_blob_no_peaks_shape(function_name, ndim, anisotropic):
+    # uniform image so no blobs will be found
+    z = np.zeros((16,) * ndim, dtype=np.float32)
+    if anisotropic:
+        max_sigma = 8 + np.arange(ndim)
+    else:
+        max_sigma = 8
+    blob_func = getattr(feature, function_name)
+    blobs = blob_func(z, max_sigma=max_sigma)
+    # z.ndim +  (z.ndim sigmas if anisotropic, only one sigma otherwise)
+    expected_shape = 2 * z.ndim if anisotropic else z.ndim + 1
+    assert blobs.shape == (0, expected_shape)
+
+
+@pytest.mark.parametrize(
+    'dtype', [np.uint8, np.float16, np.float32, np.float64]
+)
+@pytest.mark.parametrize('threshold_type', ['absolute', 'relative'])
+def test_blob_log(dtype, threshold_type):
     r2 = math.sqrt(2)
-    img = np.ones((256, 256))
+    img = np.ones((256, 256), dtype=dtype)
 
     xs, ys = disk((200, 65), 5)
     img[xs, ys] = 255
@@ -134,7 +207,18 @@ def test_blob_log():
     xs, ys = disk((100, 175), 30)
     img[xs, ys] = 255
 
-    blobs = blob_log(img, min_sigma=5, max_sigma=20, threshold=1)
+    if threshold_type == 'absolute':
+        threshold = 1
+        if img.dtype.kind != 'f':
+            # account for internal scaling to [0, 1] by img_as_float
+            threshold /= img.ptp()
+        threshold_rel = None
+    elif threshold_type == 'relative':
+        threshold = None
+        threshold_rel = 0.5
+
+    blobs = blob_log(img, min_sigma=5, max_sigma=20, threshold=threshold,
+                     threshold_rel=threshold_rel)
 
     radius = lambda x: r2 * x[2]
     s = sorted(blobs, key=radius)
@@ -165,7 +249,8 @@ def test_blob_log():
         img,
         min_sigma=5,
         max_sigma=20,
-        threshold=1,
+        threshold=threshold,
+        threshold_rel=threshold_rel,
         log_scale=True)
 
     b = s[0]
@@ -189,7 +274,7 @@ def test_blob_log():
     assert abs(radius(b) - 30) <= thresh
 
     # Testing no peaks
-    img_empty = np.zeros((100,100))
+    img_empty = np.zeros((100, 100))
     assert blob_log(img_empty).size == 0
 
 
@@ -202,9 +287,7 @@ def test_blob_log_no_warnings():
     xs, ys = disk((7, 6), 2)
     img[xs, ys] = 255
 
-    with pytest.warns(None) as records:
-        blob_log(img, max_sigma=20, num_sigma=10, threshold=.1)
-    assert len(records) == 0
+    blob_log(img, max_sigma=20, num_sigma=10, threshold=.1)
 
 
 def test_blob_log_3d():
@@ -272,8 +355,10 @@ def test_blob_log_exclude_border():
     assert blobs.shape[0] == 0, msg
 
 
-def test_blob_doh():
-    img = np.ones((512, 512), dtype=np.uint8)
+@pytest.mark.parametrize("dtype", [np.uint8, np.float16, np.float32])
+@pytest.mark.parametrize('threshold_type', ['absolute', 'relative'])
+def test_blob_doh(dtype, threshold_type):
+    img = np.ones((512, 512), dtype=dtype)
 
     xs, ys = disk((400, 130), 20)
     img[xs, ys] = 255
@@ -287,12 +372,26 @@ def test_blob_doh():
     xs, ys = disk((200, 350), 50)
     img[xs, ys] = 255
 
+    if threshold_type == 'absolute':
+        # Note: have to either scale up threshold or rescale the image to the
+        #       range [0, 1] internally.
+        threshold = 0.05
+        if img.dtype.kind == 'f':
+            # account for lack of internal scaling to [0, 1] by img_as_float
+            ptp = img.ptp()
+            threshold *= ptp ** 2
+        threshold_rel = None
+    elif threshold_type == 'relative':
+        threshold = None
+        threshold_rel = 0.5
+
     blobs = blob_doh(
         img,
         min_sigma=1,
         max_sigma=60,
         num_sigma=10,
-        threshold=.05)
+        threshold=threshold,
+        threshold_rel=threshold_rel)
 
     radius = lambda x: x[2]
     s = sorted(blobs, key=radius)
@@ -369,7 +468,7 @@ def test_blob_doh_log_scale():
 
 def test_blob_doh_no_peaks():
     # Testing no peaks
-    img_empty = np.zeros((100,100))
+    img_empty = np.zeros((100, 100))
     assert blob_doh(img_empty).size == 0
 
 
@@ -404,7 +503,7 @@ def test_blob_log_overlap_3d():
                    mode='constant')
     im3 = np.logical_or(blob1, blob2)
 
-    blobs = blob_log(im3,  min_sigma=2, max_sigma=10, overlap=0.1)
+    blobs = blob_log(im3, min_sigma=2, max_sigma=10, overlap=0.1)
     assert len(blobs) == 1
 
 
