@@ -1,17 +1,14 @@
 import numpy as np
+import pytest
+from numpy.testing import assert_almost_equal, assert_array_equal, assert_equal
 
-from skimage._shared.testing import (assert_almost_equal, assert_array_equal,
-                                     assert_equal)
 from skimage import data
 from skimage import img_as_float
 from skimage import draw
-from skimage.color import rgb2gray
-from skimage.morphology import cube, octagon
-from skimage._shared.testing import test_parallel
+from skimage._shared.utils import _supported_float_type
 from skimage._shared._warnings import expected_warnings
-from skimage._shared import testing
-import pytest
-
+from skimage._shared.testing import test_parallel
+from skimage.color import rgb2gray
 from skimage.feature import (corner_moravec, corner_harris, corner_shi_tomasi,
                              corner_subpix, peak_local_max, corner_peaks,
                              corner_kitchen_rosenfeld, corner_foerstner,
@@ -20,6 +17,7 @@ from skimage.feature import (corner_moravec, corner_harris, corner_shi_tomasi,
                              structure_tensor_eigenvalues,
                              hessian_matrix, hessian_matrix_eigvals,
                              hessian_matrix_det, shape_index)
+from skimage.morphology import cube, octagon
 
 
 @pytest.fixture
@@ -31,10 +29,13 @@ def im3d():
     return im3
 
 
-def test_structure_tensor():
-    square = np.zeros((5, 5))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_structure_tensor(dtype):
+    square = np.zeros((5, 5), dtype=dtype)
     square[2, 2] = 1
     Arr, Arc, Acc = structure_tensor(square, sigma=0.1, order='rc')
+    out_dtype = _supported_float_type(dtype)
+    assert all(a.dtype == out_dtype for a in (Arr, Arc, Acc))
     assert_array_equal(Acc, np.array([[0, 0, 0, 0, 0],
                                       [0, 1, 0, 1, 0],
                                       [0, 4, 0, 4, 0],
@@ -52,10 +53,12 @@ def test_structure_tensor():
                                       [0, 0, 0, 0, 0]]))
 
 
-def test_structure_tensor_3d():
-    cube = np.zeros((5, 5, 5))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_structure_tensor_3d(dtype):
+    cube = np.zeros((5, 5, 5), dtype=dtype)
     cube[2, 2, 2] = 1
     A_elems = structure_tensor(cube, sigma=0.1)
+    assert all(a.dtype == _supported_float_type(dtype) for a in A_elems)
     assert_equal(len(A_elems), 6)
     assert_array_equal(A_elems[0][:, 1, :], np.array([[0, 0, 0, 0, 0],
                                                       [0, 1, 4, 1, 0],
@@ -76,7 +79,7 @@ def test_structure_tensor_3d():
 
 def test_structure_tensor_3d_rc_only():
     cube = np.zeros((5, 5, 5))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         structure_tensor(cube, sigma=0.1, order='xy')
     A_elems_rc = structure_tensor(cube, sigma=0.1, order='rc')
     A_elems_none = structure_tensor(cube, sigma=0.1)
@@ -94,10 +97,28 @@ def test_structure_tensor_orders():
     assert_array_equal(A_elems_xy, A_elems_rc[::-1])
 
 
-def test_hessian_matrix():
-    square = np.zeros((5, 5))
+@pytest.mark.parametrize('ndim', [2, 3])
+def test_structure_tensor_sigma(ndim):
+    img = np.zeros((5,) * ndim)
+    img[[2] * ndim] = 1
+    A_default = structure_tensor(img, sigma=0.1, order='rc')
+    A_tuple = structure_tensor(img, sigma=(0.1,) * ndim, order='rc')
+    A_list = structure_tensor(img, sigma=[0.1] * ndim, order='rc')
+    assert_array_equal(A_tuple, A_default)
+    assert_array_equal(A_list, A_default)
+    with pytest.raises(ValueError):
+        structure_tensor(img, sigma=(0.1,) * (ndim - 1), order='rc')
+    with pytest.raises(ValueError):
+        structure_tensor(img, sigma=[0.1] * (ndim + 1), order='rc')
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_hessian_matrix(dtype):
+    square = np.zeros((5, 5), dtype=dtype)
     square[2, 2] = 4
     Hrr, Hrc, Hcc = hessian_matrix(square, sigma=0.1, order='rc')
+    out_dtype = _supported_float_type(dtype)
+    assert all(a.dtype == out_dtype for a in (Hrr, Hrc, Hcc))
     assert_almost_equal(Hrr, np.array([[0, 0,  0, 0, 0],
                                        [0, 0,  0, 0, 0],
                                        [2, 0, -2, 0, 2],
@@ -130,11 +151,14 @@ def test_hessian_matrix_3d():
                                                   [0,  0,  0,  0,  0]]))
 
 
-def test_structure_tensor_eigenvalues():
-    square = np.zeros((5, 5))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_structure_tensor_eigenvalues(dtype):
+    square = np.zeros((5, 5), dtype=dtype)
     square[2, 2] = 1
     A_elems = structure_tensor(square, sigma=0.1, order='rc')
     l1, l2 = structure_tensor_eigenvalues(A_elems)
+    out_dtype = _supported_float_type(dtype)
+    assert all(a.dtype == out_dtype for a in (l1, l2))
     assert_array_equal(l1, np.array([[0, 0, 0, 0, 0],
                                      [0, 2, 4, 2, 0],
                                      [0, 4, 0, 4, 0],
@@ -167,11 +191,14 @@ def test_structure_tensor_eigvals():
     assert_array_equal(eigvals, eigenvalues)
 
 
-def test_hessian_matrix_eigvals():
-    square = np.zeros((5, 5))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_hessian_matrix_eigvals(dtype):
+    square = np.zeros((5, 5), dtype=dtype)
     square[2, 2] = 4
     H = hessian_matrix(square, sigma=0.1, order='rc')
     l1, l2 = hessian_matrix_eigvals(H)
+    out_dtype = _supported_float_type(dtype)
+    assert all(a.dtype == out_dtype for a in (l1, l2))
     assert_almost_equal(l1, np.array([[0, 0,  2, 0, 0],
                                       [0, 1,  0, 1, 0],
                                       [2, 0, -2, 0, 2],
@@ -184,9 +211,14 @@ def test_hessian_matrix_eigvals():
                                       [0,  0,  0,  0, 0]]))
 
 
-def test_hessian_matrix_eigvals_3d(im3d):
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_hessian_matrix_eigvals_3d(im3d, dtype):
+    im3d = im3d.astype(dtype, copy=False)
     H = hessian_matrix(im3d)
     E = hessian_matrix_eigvals(H)
+    out_dtype = _supported_float_type(dtype)
+    assert all(a.dtype == out_dtype for a in E)
+
     # test descending order:
     e0, e1, e2 = E
     assert np.all(e0 >= e1) and np.all(e1 >= e2)
@@ -212,8 +244,11 @@ def test_hessian_matrix_det():
     assert_almost_equal(det, 0, decimal=3)
 
 
-def test_hessian_matrix_det_3d(im3d):
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_hessian_matrix_det_3d(im3d, dtype):
+    im3d = im3d.astype(dtype, copy=False)
     D = hessian_matrix_det(im3d)
+    assert D.dtype == _supported_float_type(dtype)
     D0 = D[D.shape[0] // 2]
     row_center, col_center = np.array(D0.shape) // 2
     # testing in 3D is hard. We test this by showing that you get the
@@ -273,6 +308,32 @@ def test_square_image():
     assert len(results) == 1
 
 
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+@pytest.mark.parametrize(
+    'func',
+    [
+        corner_moravec,
+        corner_harris,
+        corner_shi_tomasi,
+        corner_kitchen_rosenfeld,
+    ]
+)
+def test_corner_dtype(dtype, func):
+    im = np.zeros((50, 50), dtype=dtype)
+    im[:25, :25] = 1.
+    out_dtype = _supported_float_type(dtype)
+    corners = func(im)
+    assert corners.dtype == out_dtype
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_corner_foerstner_dtype(dtype):
+    im = np.zeros((50, 50), dtype=dtype)
+    im[:25, :25] = 1.
+    out_dtype = _supported_float_type(dtype)
+    assert all(arr.dtype == out_dtype for arr in corner_foerstner(im))
+
+
 def test_noisy_square_image():
     im = np.zeros((50, 50)).astype(float)
     im[:25, :25] = 1.
@@ -344,13 +405,15 @@ def test_rotated_img():
     assert (np.sort(results[1]) == np.sort(results_rotated[0])).all()
 
 
-def test_subpix_edge():
-    img = np.zeros((50, 50))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_subpix_edge(dtype):
+    img = np.zeros((50, 50), dtype=dtype)
     img[:25, :25] = 255
     img[25:, 25:] = 255
     corner = peak_local_max(corner_harris(img),
                             min_distance=10, threshold_rel=0, num_peaks=1)
     subpix = corner_subpix(img, corner)
+    assert subpix.dtype == _supported_float_type(dtype)
     assert_array_equal(subpix[0], (24.5, 24.5))
 
 
@@ -395,7 +458,9 @@ def test_subpix_border():
 def test_num_peaks():
     """For a bunch of different values of num_peaks, check that
     peak_local_max returns exactly the right amount of peaks. Test
-    is run on the astronaut image in order to produce a sufficient number of corners"""
+    is run on the astronaut image in order to produce a sufficient number of
+    corners.
+    """
 
     img_corners = corner_harris(rgb2gray(data.astronaut()))
 
@@ -444,7 +509,7 @@ def test_blank_image_nans():
 
 def test_corner_fast_image_unsupported_error():
     img = np.zeros((20, 20, 3))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         corner_fast(img)
 
 
@@ -495,7 +560,7 @@ def test_corner_fast_astronaut():
 
 def test_corner_orientations_image_unsupported_error():
     img = np.zeros((20, 20, 3))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         corner_orientations(
             img,
             np.asarray([[7, 7]]), np.ones((3, 3)))
@@ -503,7 +568,7 @@ def test_corner_orientations_image_unsupported_error():
 
 def test_corner_orientations_even_shape_error():
     img = np.zeros((20, 20))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         corner_orientations(
             img,
             np.asarray([[7, 7]]), np.ones((4, 4)))
@@ -537,12 +602,14 @@ def test_corner_orientations_astronaut():
     assert_almost_equal(actual, expected)
 
 
-def test_corner_orientations_square():
-    square = np.zeros((12, 12))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_corner_orientations_square(dtype):
+    square = np.zeros((12, 12), dtype=dtype)
     square[3:9, 3:9] = 1
     corners = corner_peaks(corner_fast(square, 9),
                            min_distance=1, threshold_rel=0)
     actual_orientations = corner_orientations(square, corners, octagon(3, 2))
+    assert actual_orientations.dtype == _supported_float_type(dtype)
     actual_orientations_degrees = np.rad2deg(actual_orientations)
     expected_orientations_degree = np.array([45, 135, -45, -135])
     assert_array_equal(actual_orientations_degrees,
