@@ -1,27 +1,48 @@
 import numpy as np
-from skimage import data
-from skimage import filters
+import pytest
+from numpy.testing import assert_, assert_allclose, assert_array_almost_equal
+
+from skimage import data, filters
+from skimage._shared.utils import _supported_float_type
 from skimage.filters.edges import _mask_filter_result
 
-from skimage._shared import testing
-from skimage._shared.testing import (assert_array_almost_equal,
-                                     assert_, assert_allclose)
 
-
-def test_roberts_zeros():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_roberts_zeros(dtype):
     """Roberts' filter on an array of all zeros."""
-    result = filters.roberts(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.roberts(np.zeros((10, 10), dtype=dtype),
+                             np.ones((10, 10), bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert (np.all(result == 0))
 
 
-def test_roberts_diagonal1():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_roberts_diagonal1(dtype):
     """Roberts' filter on a diagonal edge should be a diagonal line."""
-    image = np.tri(10, 10, 0)
+    image = np.tri(10, 10, 0, dtype=dtype)
     expected = ~(np.tri(10, 10, -1).astype(bool) |
                  np.tri(10, 10, -2).astype(bool).transpose())
     expected[-1, -1] = 0  # due to 'reflect' & image shape, last pixel not edge
-    result = filters.roberts(image).astype(bool)
-    assert_array_almost_equal(result, expected)
+    result = filters.roberts(image)
+    assert result.dtype == _supported_float_type(dtype)
+    assert_array_almost_equal(result.astype(bool), expected)
+
+
+@pytest.mark.parametrize(
+    'function_name',
+    ['farid', 'laplace', 'prewitt', 'roberts', 'scharr', 'sobel']
+)
+def test_int_rescaling(function_name):
+    """Basic test that uint8 inputs get rescaled from [0, 255] to [0, 1.]
+
+    The output of any of these filters should be within roughly a factor of
+    two of the input range. For integer inputs, rescaling to floats in
+    [0.0, 1.0] should occur, so just verify outputs are not > 2.0.
+    """
+    img = data.coins()[:128, :128]
+    func = getattr(filters, function_name)
+    filtered = func(img)
+    assert filtered.max() <= 2.0
 
 
 def test_roberts_diagonal2():
@@ -40,10 +61,12 @@ def test_sobel_zeros():
     assert (np.all(result == 0))
 
 
-def test_sobel_mask():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_sobel_mask(dtype):
     """Sobel on a masked array should be zero."""
-    result = filters.sobel(np.random.uniform(size=(10, 10)),
+    result = filters.sobel(np.random.uniform(size=(10, 10)).astype(dtype),
                            np.zeros((10, 10), dtype=bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert (np.all(result == 0))
 
 
@@ -134,10 +157,12 @@ def test_scharr_zeros():
     assert (np.all(result < 1e-16))
 
 
-def test_scharr_mask():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_scharr_mask(dtype):
     """Scharr on a masked array should be zero."""
-    result = filters.scharr(np.random.uniform(size=(10, 10)),
+    result = filters.scharr(np.random.uniform(size=(10, 10)).astype(dtype),
                             np.zeros((10, 10), dtype=bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert_allclose(result, 0)
 
 
@@ -231,10 +256,12 @@ def test_prewitt_zeros():
     assert_allclose(result, 0)
 
 
-def test_prewitt_mask():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_prewitt_mask(dtype):
     """Prewitt on a masked array should be zero."""
-    result = filters.prewitt(np.random.uniform(size=(10, 10)),
+    result = filters.prewitt(np.random.uniform(size=(10, 10)).astype(dtype),
                              np.zeros((10, 10), dtype=bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert_allclose(np.abs(result), 0)
 
 
@@ -339,13 +366,15 @@ def test_laplace_zeros():
     assert_allclose(result, check_result)
 
 
-def test_laplace_mask():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_laplace_mask(dtype):
     """Laplace on a masked array should be zero."""
     # Create a synthetic 2D image
-    image = np.zeros((9, 9))
+    image = np.zeros((9, 9), dtype=dtype)
     image[3:-3, 3:-3] = 1
     # Define the mask
     result = filters.laplace(image, ksize=3, mask=np.zeros((9, 9), dtype=bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert (np.all(result == 0))
 
 
@@ -356,10 +385,12 @@ def test_farid_zeros():
     assert (np.all(result == 0))
 
 
-def test_farid_mask():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_farid_mask(dtype):
     """Farid on a masked array should be zero."""
-    result = filters.farid(np.random.uniform(size=(10, 10)),
+    result = filters.farid(np.random.uniform(size=(10, 10)).astype(dtype),
                            mask=np.zeros((10, 10), dtype=bool))
+    assert result.dtype == _supported_float_type(dtype)
     assert (np.all(result == 0))
 
 
@@ -446,8 +477,9 @@ def test_farid_v_horizontal():
     assert_allclose(result, 0, atol=1e-10)
 
 
-@testing.parametrize("grad_func", (filters.prewitt_h, filters.sobel_h,
-                                   filters.scharr_h))
+@pytest.mark.parametrize(
+    "grad_func", (filters.prewitt_h, filters.sobel_h, filters.scharr_h)
+)
 def test_horizontal_mask_line(grad_func):
     """Horizontal edge filters mask pixels surrounding input mask."""
     vgrad, _ = np.mgrid[:1:11j, :1:11j]  # vertical gradient with spacing 0.1
@@ -464,8 +496,10 @@ def test_horizontal_mask_line(grad_func):
     assert_allclose(result, expected)
 
 
-@testing.parametrize("grad_func", (
-    filters.prewitt_v, filters.sobel_v, filters.scharr_v))
+@pytest.mark.parametrize(
+    "grad_func",
+    (filters.prewitt_v, filters.sobel_v, filters.scharr_v)
+)
 def test_vertical_mask_line(grad_func):
     """Vertical edge filters mask pixels surrounding input mask."""
     _, hgrad = np.mgrid[:1:11j, :1:11j]  # horizontal gradient with spacing 0.1
@@ -530,7 +564,7 @@ MAX_SCHARR_ND = np.array([
 ]).astype(float)
 
 
-@testing.parametrize(
+@pytest.mark.parametrize(
     ('func', 'max_edge'),
     [(filters.prewitt, MAX_SOBEL_ND),
      (filters.sobel, MAX_SOBEL_ND),
@@ -539,19 +573,19 @@ MAX_SCHARR_ND = np.array([
 def test_3d_edge_filters(func, max_edge):
     blobs = data.binary_blobs(length=128, n_dim=3)
     edges = func(blobs)
-    testing.assert_allclose(np.max(edges), func(max_edge)[1, 1, 1])
+    assert_allclose(np.max(edges), func(max_edge)[1, 1, 1])
 
 
-@testing.parametrize(
+@pytest.mark.parametrize(
     'func', (filters.prewitt, filters.sobel, filters.scharr)
 )
 def test_3d_edge_filters_single_axis(func):
     blobs = data.binary_blobs(length=128, n_dim=3)
     edges0 = func(blobs, axis=0)
-    testing.assert_allclose(np.max(edges0), func(MAX_SOBEL_0, axis=0)[1, 1, 1])
+    assert_allclose(np.max(edges0), func(MAX_SOBEL_0, axis=0)[1, 1, 1])
 
 
-@testing.parametrize(
+@pytest.mark.parametrize(
     'detector',
     [filters.sobel, filters.scharr, filters.prewitt,
      filters.roberts, filters.farid]
@@ -563,6 +597,7 @@ def test_range(detector):
     assert_(
         out.min() >= 0, f'Minimum of `{detector.__name__}` is smaller than 0.'
     )
+
     assert_(
         out.max() <= 1, f'Maximum of `{detector.__name__}` is larger than 1.'
     )
