@@ -1,11 +1,13 @@
-import pytest
 import numpy as np
+import pytest
 from numpy.testing import assert_allclose, assert_array_less, assert_equal
-from skimage.filters import meijering, sato, frangi, hessian
-from skimage.data import camera, retina
-from skimage.util import crop, invert
+
+from skimage import img_as_float
+from skimage._shared.utils import _supported_float_type
 from skimage.color import rgb2gray
-from skimage._shared._warnings import expected_warnings
+from skimage.data import camera, retina
+from skimage.filters import frangi, hessian, meijering, sato
+from skimage.util import crop, invert
 
 
 def test_2d_null_matrix():
@@ -31,11 +33,13 @@ def test_2d_null_matrix():
 
 def test_3d_null_matrix():
 
-    a_black = np.zeros((3, 3, 3)).astype(np.uint8)
+    # Note: last axis intentionally not size 3 to avoid 2D+RGB autodetection
+    #       warning from an internal call to `skimage.filters.gaussian`.
+    a_black = np.zeros((3, 3, 5)).astype(np.uint8)
     a_white = invert(a_black)
 
-    zeros = np.zeros((3, 3, 3))
-    ones = np.ones((3, 3, 3))
+    zeros = np.zeros((3, 3, 5))
+    ones = np.ones((3, 3, 5))
 
     assert_allclose(meijering(a_black, black_ridges=True), zeros, atol=1e-1)
     assert_allclose(meijering(a_white, black_ridges=False), zeros, atol=1e-1)
@@ -136,7 +140,9 @@ def test_2d_linearity():
 
 def test_3d_linearity():
 
-    a_black = np.ones((3, 3, 3)).astype(np.uint8)
+    # Note: last axis intentionally not size 3 to avoid 2D+RGB autodetection
+    #       warning from an internal call to `skimage.filters.gaussian`.
+    a_black = np.ones((3, 3, 5)).astype(np.uint8)
     a_white = invert(a_black)
 
     assert_allclose(meijering(1 * a_black, black_ridges=True),
@@ -187,14 +193,21 @@ def test_2d_cropped_camera_image():
                     ones, atol=1 - 1e-7)
 
 
+@pytest.mark.parametrize('func', [meijering, sato, frangi, hessian])
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_ridge_output_dtype(func, dtype):
+    img = img_as_float(camera()).astype(dtype, copy=False)
+    assert func(img).dtype == _supported_float_type(img.dtype)
+
+
 def test_3d_cropped_camera_image():
 
     a_black = crop(camera(), ((200, 212), (100, 312)))
-    a_black = np.dstack([a_black, a_black, a_black])
+    a_black = np.stack([a_black] * 5, axis=-1)
     a_white = invert(a_black)
 
-    zeros = np.zeros((100, 100, 3))
-    ones = np.ones((100, 100, 3))
+    zeros = np.zeros(a_black.shape)
+    ones = np.ones(a_black.shape)
 
     assert_allclose(meijering(a_black, black_ridges=True),
                     meijering(a_white, black_ridges=False))
@@ -234,16 +247,3 @@ def test_border_management(func, tol):
     assert abs(full_mean - inside_mean) < tol
     assert abs(full_mean - border_mean) < tol
     assert abs(inside_mean - border_mean) < tol
-
-
-@pytest.mark.parametrize('func', [sato, hessian])
-def test_border_warning(func):
-    img = rgb2gray(retina()[300:500, 700:900])
-
-    with expected_warnings(["implicitly used 'constant' as the border mode"]):
-        func(img, sigmas=[1])
-
-
-if __name__ == "__main__":
-    from numpy import testing
-    testing.run_module_suite()
