@@ -1,20 +1,18 @@
 __all__ = ['imread', 'imsave']
 
-from distutils.version import LooseVersion
-import warnings
 import numpy as np
+from packaging import version
 from PIL import Image, __version__ as pil_version
 
 from ...util import img_as_ubyte, img_as_uint
 
-# Check CVE-2020-10379
-from distutils.version import LooseVersion
-if LooseVersion(pil_version) < LooseVersion('7.1.0'):
+# Check CVE-2021-27921 and others
+if version.parse(pil_version) < version.parse('8.1.2'):
     from warnings import warn
-    warn('Your installed pillow version is < 7.1.0. '
-         'Several security issues (CVE-2020-11538, '
-         'CVE-2020-10379, CVE-2020-10994, CVE-2020-10177) '
-         'have been fixed in pillow 7.1.0 or higher. '
+    warn('Your installed pillow version is < 8.1.2. '
+         'Several security issues (CVE-2021-27921, '
+         'CVE-2021-25290, CVE-2021-25291, CVE-2021-25293, '
+         'and more) have been fixed in pillow 8.1.2 or higher. '
          'We recommend to upgrade this library.',
          stacklevel=2)
 
@@ -49,11 +47,6 @@ def imread(fname, dtype=None, img_num=None, **kwargs):
             return pil_to_ndarray(im, dtype=dtype, img_num=img_num)
     else:
         im = Image.open(fname)
-        if im.format == 'MPO' and LooseVersion(pil_version) < '6.0.0':
-            warnings.warn("You are trying to read a MPO image. "
-                          "To ensure a good support of this format, "
-                          "please upgrade pillow to 6.0.0 version or later.",
-                          stacklevel=2)
         return pil_to_ndarray(im, dtype=dtype, img_num=img_num)
 
 
@@ -160,7 +153,7 @@ def _palette_is_grayscale(pil_image):
     if pil_image.mode != 'P':
         raise ValueError('pil_image.mode must be equal to "P".')
     # get palette as an array with R, G, B columns
-    palette = np.asarray(pil_image.getpalette()).reshape((256, 3))
+    palette = np.asarray(pil_image.getpalette()).reshape((-1, 3))
     # Not all palette colors are used; unused colors have junk values.
     start, stop = pil_image.getextrema()
     valid_palette = palette[start:stop + 1]
