@@ -1,11 +1,11 @@
-from skimage._shared._warnings import expected_warnings
-from skimage._shared.testing import assert_equal, assert_almost_equal
-from skimage._shared import testing
 import numpy as np
+import pytest
+from numpy.testing import assert_equal, assert_almost_equal
 
 from skimage import data
+from skimage._shared._warnings import expected_warnings
 from skimage.metrics import (peak_signal_noise_ratio, normalized_root_mse,
-                             mean_squared_error)
+                             mean_squared_error, normalized_mutual_information)
 
 
 np.random.seed(5)
@@ -38,35 +38,48 @@ def test_PSNR_vs_IPOL():
     assert_almost_equal(p, p_IPOL, decimal=4)
 
 
-def test_PSNR_float():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_PSNR_float(dtype):
     p_uint8 = peak_signal_noise_ratio(cam, cam_noisy)
-    p_float64 = peak_signal_noise_ratio(cam / 255., cam_noisy / 255.,
-                                        data_range=1)
-    assert_almost_equal(p_uint8, p_float64, decimal=5)
+    camf = (cam / 255.).astype(dtype, copy=False)
+    camf_noisy = (cam_noisy / 255.).astype(dtype, copy=False)
+    p_float64 = peak_signal_noise_ratio(camf, camf_noisy, data_range=1)
+    assert p_float64.dtype == np.float64
+    decimal = 3 if dtype == np.float16 else 5
+    assert_almost_equal(p_uint8, p_float64, decimal=decimal)
 
     # mixed precision inputs
     p_mixed = peak_signal_noise_ratio(cam / 255., np.float32(cam_noisy / 255.),
                                       data_range=1)
-    assert_almost_equal(p_mixed, p_float64, decimal=5)
+
+    assert_almost_equal(p_mixed, p_float64, decimal=decimal)
 
     # mismatched dtype results in a warning if data_range is unspecified
     with expected_warnings(['Inputs have mismatched dtype']):
         p_mixed = peak_signal_noise_ratio(cam / 255.,
                                           np.float32(cam_noisy / 255.))
-    assert_almost_equal(p_mixed, p_float64, decimal=5)
+    assert_almost_equal(p_mixed, p_float64, decimal=decimal)
+
+    # mismatched dtype results in a warning if data_range is unspecified
+    with expected_warnings(['Inputs have mismatched dtype']):
+        p_mixed = peak_signal_noise_ratio(cam / 255.,
+                                          np.float32(cam_noisy / 255.))
+    assert_almost_equal(p_mixed, p_float64, decimal=decimal)
 
 
 def test_PSNR_errors():
     # shape mismatch
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         peak_signal_noise_ratio(cam, cam[:-1, :])
 
 
-def test_NRMSE():
-    x = np.ones(4)
-    y = np.asarray([0., 2., 2., 2.])
-    assert_equal(normalized_root_mse(y, x, normalization='mean'),
-                 1 / np.mean(y))
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_NRMSE(dtype):
+    x = np.ones(4, dtype=dtype)
+    y = np.asarray([0., 2., 2., 2.], dtype=dtype)
+    nrmse = normalized_root_mse(y, x, normalization='mean')
+    assert nrmse.dtype == np.float64
+    assert_equal(nrmse, 1 / np.mean(y))
     assert_equal(normalized_root_mse(y, x, normalization='euclidean'),
                  1 / np.sqrt(3))
     assert_equal(normalized_root_mse(y, x, normalization='min-max'),
@@ -90,8 +103,37 @@ def test_NRMSE_no_int_overflow():
 def test_NRMSE_errors():
     x = np.ones(4)
     # shape mismatch
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         normalized_root_mse(x[:-1], x)
     # invalid normalization name
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         normalized_root_mse(x, x, normalization='foo')
+
+
+def test_nmi():
+    assert_almost_equal(normalized_mutual_information(cam, cam), 2)
+    assert (normalized_mutual_information(cam, cam_noisy)
+            < normalized_mutual_information(cam, cam))
+
+
+def test_nmi_different_sizes():
+    assert normalized_mutual_information(cam[:, :400], cam[:400, :]) > 1
+
+
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_nmi_random(dtype):
+    rng = np.random.default_rng()
+    random1 = rng.random((100, 100)).astype(dtype)
+    random2 = rng.random((100, 100)).astype(dtype)
+    nmi = normalized_mutual_information(random1, random2, bins=10)
+    assert nmi.dtype == np.float64
+    assert_almost_equal(nmi, 1, decimal=2)
+
+
+def test_nmi_random_3d():
+    random1, random2 = np.random.random((2, 10, 100, 100))
+    assert_almost_equal(
+        normalized_mutual_information(random1, random2, bins=10),
+        1,
+        decimal=2,
+    )
