@@ -11,9 +11,9 @@ Soille, P. (2003). Morphological Image Analysis: Principles and Applications
 """
 import numpy as np
 
+from .._shared.utils import deprecate_kwarg, warn
 from ..util import dtype_limits, invert, crop
-from .._shared.utils import warn
-from . import greyreconstruct, _util
+from . import grayreconstruct, _util
 from ._extrema_cy import _local_maxima
 
 
@@ -45,11 +45,13 @@ def _subtract_constant_clip(image, const_value):
     return(result)
 
 
-def h_maxima(image, h, selem=None):
+@deprecate_kwarg(kwarg_mapping={'selem': 'footprint'}, removed_version="1.0",
+                 deprecated_version="0.19")
+def h_maxima(image, h, footprint=None):
     """Determine all maxima of the image with height >= h.
 
     The local maxima are defined as connected sets of pixels with equal
-    grey level strictly greater than the grey level of all pixels in direct
+    gray level strictly greater than the gray level of all pixels in direct
     neighborhood of the set.
 
     A local maximum M of height h is a local maximum for which
@@ -67,7 +69,7 @@ def h_maxima(image, h, selem=None):
         The input image for which the maxima are to be calculated.
     h : unsigned integer
         The minimal height of all extracted maxima.
-    selem : ndarray, optional
+    footprint : ndarray, optional
         The neighborhood expressed as an n-D array of 1's and 0's.
         Default is the ball of radius 1 according to the maximum norm
         (i.e. a 3x3 square for 2D images, a 3x3x3 cube for 3D images, etc.)
@@ -75,11 +77,11 @@ def h_maxima(image, h, selem=None):
     Returns
     -------
     h_max : ndarray
-       The local maxima of height >= h and the global maxima.
-       The resulting image is a binary image, where pixels belonging to
-       the determined maxima take value 1, the others take value 0.
+        The local maxima of height >= h and the global maxima.
+        The resulting image is a binary image, where pixels belonging to
+        the determined maxima take value 1, the others take value 0.
 
-    See also
+    See Also
     --------
     skimage.morphology.extrema.h_minima
     skimage.morphology.extrema.local_maxima
@@ -167,17 +169,20 @@ def h_maxima(image, h, selem=None):
     else:
         shifted_img = _subtract_constant_clip(image, h)
 
-    rec_img = greyreconstruct.reconstruction(shifted_img, image,
-                                             method='dilation', selem=selem)
+    rec_img = grayreconstruct.reconstruction(shifted_img, image,
+                                             method='dilation',
+                                             footprint=footprint)
     residue_img = image - rec_img
     return (residue_img >= h).astype(np.uint8)
 
 
-def h_minima(image, h, selem=None):
+@deprecate_kwarg(kwarg_mapping={'selem': 'footprint'}, removed_version="1.0",
+                 deprecated_version="0.19")
+def h_minima(image, h, footprint=None):
     """Determine all minima of the image with depth >= h.
 
     The local minima are defined as connected sets of pixels with equal
-    grey level strictly smaller than the grey levels of all pixels in direct
+    gray level strictly smaller than the gray levels of all pixels in direct
     neighborhood of the set.
 
     A local minimum M of depth h is a local minimum for which
@@ -195,7 +200,7 @@ def h_minima(image, h, selem=None):
         The input image for which the minima are to be calculated.
     h : unsigned integer
         The minimal depth of all extracted minima.
-    selem : ndarray, optional
+    footprint : ndarray, optional
         The neighborhood expressed as an n-D array of 1's and 0's.
         Default is the ball of radius 1 according to the maximum norm
         (i.e. a 3x3 square for 2D images, a 3x3x3 cube for 3D images, etc.)
@@ -203,11 +208,11 @@ def h_minima(image, h, selem=None):
     Returns
     -------
     h_min : ndarray
-       The local minima of depth >= h and the global minima.
-       The resulting image is a binary image, where pixels belonging to
-       the determined minima take value 1, the others take value 0.
+        The local minima of depth >= h and the global minima.
+        The resulting image is a binary image, where pixels belonging to
+        the determined minima take value 1, the others take value 0.
 
-    See also
+    See Also
     --------
     skimage.morphology.extrema.h_maxima
     skimage.morphology.extrema.local_maxima
@@ -263,13 +268,16 @@ def h_minima(image, h, selem=None):
     else:
         shifted_img = _add_constant_clip(image, h)
 
-    rec_img = greyreconstruct.reconstruction(shifted_img, image,
-                                             method='erosion', selem=selem)
+    rec_img = grayreconstruct.reconstruction(shifted_img, image,
+                                             method='erosion',
+                                             footprint=footprint)
     residue_img = rec_img - image
     return (residue_img >= h).astype(np.uint8)
 
 
-def local_maxima(image, selem=None, connectivity=None, indices=False,
+@deprecate_kwarg(kwarg_mapping={'selem': 'footprint'}, removed_version="1.0",
+                 deprecated_version="0.19")
+def local_maxima(image, footprint=None, connectivity=None, indices=False,
                  allow_borders=True):
     """Find local maxima of n-dimensional array.
 
@@ -281,17 +289,17 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
     ----------
     image : ndarray
         An n-dimensional array.
-    selem : ndarray, optional
-        A structuring element used to determine the neighborhood of each
-        evaluated pixel (``True`` denotes a connected pixel). It must be a
-        boolean array and have the same number of dimensions as `image`. If
-        neither `selem` nor `connectivity` are given, all adjacent pixels are
-        considered as part of the neighborhood.
+    footprint : ndarray, optional
+        The footprint (structuring element) used to determine the neighborhood
+        of each evaluated pixel (``True`` denotes a connected pixel). It must
+        be a boolean array and have the same number of dimensions as `image`.
+        If neither `footprint` nor `connectivity` are given, all adjacent
+        pixels are considered as part of the neighborhood.
     connectivity : int, optional
         A number used to determine the neighborhood of each evaluated pixel.
         Adjacent pixels whose squared distance from the center is less than or
         equal to `connectivity` are considered neighbors. Ignored if
-        `selem` is not None.
+        `footprint` is not None.
     indices : bool, optional
         If True, the output will be a tuple of one-dimensional arrays
         representing the indices of local maxima in each dimension. If False,
@@ -388,7 +396,7 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
     if allow_borders:
         # Ensure that local maxima are always at least one smaller sample away
         # from the image border
-        image = _util._fast_pad(image, image.min())
+        image = np.pad(image, 1, mode='constant', constant_values=image.min())
 
     # Array of flags used to store the state of each pixel during evaluation.
     # See _extrema_cy.pyx for their meaning
@@ -397,16 +405,17 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
 
     if any(s < 3 for s in image.shape):
         # Warn and skip if any dimension is smaller than 3
-        # -> no maxima can exist & structuring element can't be applied
+        # -> no maxima can exist & footprint can't be applied
         warn(
             "maxima can't exist for an image with any dimension smaller 3 "
             "if borders aren't allowed",
             stacklevel=3
         )
     else:
-        selem = _util._resolve_neighborhood(selem, connectivity, image.ndim)
+        footprint = _util._resolve_neighborhood(footprint, connectivity,
+                                                image.ndim)
         neighbor_offsets = _util._offsets_to_raveled_neighbors(
-            image.shape, selem, center=((1,) * image.ndim)
+            image.shape, footprint, center=((1,) * image.ndim)
         )
 
         try:
@@ -432,7 +441,9 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
         return flags.view(bool)
 
 
-def local_minima(image, selem=None, connectivity=None, indices=False,
+@deprecate_kwarg(kwarg_mapping={'selem': 'footprint'}, removed_version="1.0",
+                 deprecated_version="0.19")
+def local_minima(image, footprint=None, connectivity=None, indices=False,
                  allow_borders=True):
     """Find local minima of n-dimensional array.
 
@@ -444,17 +455,17 @@ def local_minima(image, selem=None, connectivity=None, indices=False,
     ----------
     image : ndarray
         An n-dimensional array.
-    selem : ndarray, optional
-        A structuring element used to determine the neighborhood of each
-        evaluated pixel (``True`` denotes a connected pixel). It must be a
-        boolean array and have the same number of dimensions as `image`. If
-        neither `selem` nor `connectivity` are given, all adjacent pixels are
-        considered as part of the neighborhood.
+    footprint : ndarray, optional
+        The footprint (structuring element) used to determine the neighborhood
+        of each evaluated pixel (``True`` denotes a connected pixel). It must
+        be a boolean array and have the same number of dimensions as `image`.
+        If neither `footprint` nor `connectivity` are given, all adjacent
+        pixels are considered as part of the neighborhood.
     connectivity : int, optional
         A number used to determine the neighborhood of each evaluated pixel.
         Adjacent pixels whose squared distance from the center is less than or
         equal to `connectivity` are considered neighbors. Ignored if
-        `selem` is not None.
+        `footprint` is not None.
     indices : bool, optional
         If True, the output will be a tuple of one-dimensional arrays
         representing the indices of local minima in each dimension. If False,
@@ -535,7 +546,7 @@ def local_minima(image, selem=None, connectivity=None, indices=False,
     """
     return local_maxima(
         image=invert(image),
-        selem=selem,
+        footprint=footprint,
         connectivity=connectivity,
         indices=indices,
         allow_borders=allow_borders
