@@ -1,26 +1,22 @@
 import numpy as np
-from scipy.ndimage import fourier_shift
-from skimage._shared import testing
-from skimage._shared.testing import assert_equal, fetch, expected_warnings
-from skimage.data import camera, stereo_motorcycle
-from skimage.registration import phase_cross_correlation
+import pytest
+from numpy.testing import (assert_almost_equal, assert_array_almost_equal,
+                           assert_array_equal, assert_array_less, assert_equal)
+from scipy.ndimage import fourier_shift, shift as real_shift
+import scipy.fft as fft
+
+from skimage._shared.testing import fetch
+from skimage._shared.utils import _supported_float_type
+from skimage.data import camera, brain
+
+
+from skimage.io import imread
 from skimage.registration._masked_phase_cross_correlation import (
     _masked_phase_cross_correlation as masked_register_translation,
     cross_correlate_masked)
-from skimage.io import imread
-from skimage._shared.fft import fftmodule as fft
-from skimage.feature import masked_register_translation as _deprecated
+from skimage.registration import phase_cross_correlation
 
 
-def test_detrecated_masked_register_translation():
-    reference_image, moving_image, _ = stereo_motorcycle()
-    ref_mask = np.random.choice(
-        [True, False], reference_image.shape, p=[3 / 4, 1 / 4])
-    with expected_warnings(["Function ``masked_register_translation``"]):
-        assert_equal(_deprecated(reference_image, moving_image, ref_mask),
-                     phase_cross_correlation(reference_image, moving_image,
-                                             reference_mask=ref_mask))
-
 def test_masked_registration_vs_phase_cross_correlation():
     """masked_register_translation should give the same results as
     phase_cross_correlation in the case of trivial masks."""
@@ -63,6 +59,25 @@ def test_masked_registration_random_masks():
     assert_equal(measured_shift, -np.array(shift))
 
 
+def test_masked_registration_3d_contiguous_mask():
+    """masked_register_translation should be able to register translations
+    between volumes with contiguous masks."""
+    ref_vol = brain()[:, ::2, ::2]
+
+    offset = (1, -5, 10)
+
+    # create square mask
+    ref_mask = np.zeros_like(ref_vol, dtype=bool)
+    ref_mask[:-2, 75:100, 75:100] = True
+    ref_shifted = real_shift(ref_vol, offset)
+
+    measured_offset = masked_register_translation(
+        ref_vol, ref_shifted, reference_mask=ref_mask, moving_mask=ref_mask
+    )
+
+    assert_equal(offset, -np.array(measured_offset))
+
+
 def test_masked_registration_random_masks_non_equal_sizes():
     """masked_register_translation should be able to register
     translations between images that are not the same size even
@@ -103,11 +118,9 @@ def test_masked_registration_padfield_data():
     for xi, yi in shifts:
 
         fixed_image = imread(
-            fetch('registration/tests/data/OriginalX{:d}Y{:d}.png'
-                  ''.format(xi, yi)))
+            fetch(f'registration/tests/data/OriginalX{xi}Y{yi}.png'))
         moving_image = imread(
-            fetch('registration/tests/data/TransformedX{:d}Y{:d}.png'
-                  ''.format(xi, yi)))
+            fetch(f'registration/tests/data/TransformedX{xi}Y{yi}.png'))
 
         # Valid pixels are 1
         fixed_mask = (fixed_image != 0)
@@ -123,7 +136,8 @@ def test_masked_registration_padfield_data():
         assert_equal((shift_x, shift_y), (-xi, yi))
 
 
-def test_cross_correlate_masked_output_shape():
+@pytest.mark.parametrize('dtype', [np.float16, np.float32, np.float64])
+def test_cross_correlate_masked_output_shape(dtype):
     """Masked normalized cross-correlation should return a shape
     of N + M + 1 for each transform axis."""
     shape1 = (15, 4, 5)
@@ -131,19 +145,24 @@ def test_cross_correlate_masked_output_shape():
     expected_full_shape = tuple(np.array(shape1) + np.array(shape2) - 1)
     expected_same_shape = shape1
 
-    arr1 = np.zeros(shape1)
-    arr2 = np.zeros(shape2)
+    arr1 = np.zeros(shape1, dtype=dtype)
+    arr2 = np.zeros(shape2, dtype=dtype)
     # Trivial masks
     m1 = np.ones_like(arr1)
     m2 = np.ones_like(arr2)
 
+    float_dtype = _supported_float_type(dtype)
+
     full_xcorr = cross_correlate_masked(
         arr1, arr2, m1, m2, axes=(0, 1, 2), mode='full')
     assert_equal(full_xcorr.shape, expected_full_shape)
+    assert full_xcorr.dtype == float_dtype
 
     same_xcorr = cross_correlate_masked(
         arr1, arr2, m1, m2, axes=(0, 1, 2), mode='same')
     assert_equal(same_xcorr.shape, expected_same_shape)
+    assert same_xcorr.dtype == float_dtype
+
 
 
 def test_cross_correlate_masked_test_against_mismatched_dimensions():
@@ -159,7 +178,8 @@ def test_cross_correlate_masked_test_against_mismatched_dimensions():
     m1 = np.ones_like(arr1)
     m2 = np.ones_like(arr2)
 
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
+
         cross_correlate_masked(arr1, arr2, m1, m2, axes=(1, 2))
 
 
@@ -187,8 +207,8 @@ def test_cross_correlate_masked_output_range():
     # No assert array less or equal, so we add an eps
     # Also could not find an `assert_array_greater`, Use (-xcorr) instead
     eps = np.finfo(float).eps
-    testing.assert_array_less(xcorr, 1 + eps)
-    testing.assert_array_less(-xcorr, 1 + eps)
+    assert_array_less(xcorr, 1 + eps)
+    assert_array_less(-xcorr, 1 + eps)
 
 
 def test_cross_correlate_masked_side_effects():
@@ -222,7 +242,7 @@ def test_cross_correlate_masked_over_axes():
     m2 = np.random.choice([True, False], arr2.shape)
 
     # Loop over last axis
-    with_loop = np.empty_like(arr1, dtype=np.complex)
+    with_loop = np.empty_like(arr1, dtype=complex)
     for index in range(arr1.shape[-1]):
         with_loop[:, :, index] = cross_correlate_masked(arr1[:, :, index],
                                                         arr2[:, :, index],
@@ -234,7 +254,7 @@ def test_cross_correlate_masked_over_axes():
     over_axes = cross_correlate_masked(
         arr1, arr2, m1, m2, axes=(0, 1), mode='same')
 
-    testing.assert_array_almost_equal(with_loop, over_axes)
+    assert_array_almost_equal(with_loop, over_axes)
 
 
 def test_cross_correlate_masked_autocorrelation_trivial_masks():
@@ -254,5 +274,6 @@ def test_cross_correlate_masked_autocorrelation_trivial_masks():
     max_index = np.unravel_index(np.argmax(xcorr), xcorr.shape)
 
     # Autocorrelation should have maximum in center of array
-    testing.assert_almost_equal(xcorr.max(), 1)
-    testing.assert_array_equal(max_index, np.array(arr1.shape) / 2)
+    # uint8 inputs will be processed in float32, so reduce decimal to 5
+    assert_almost_equal(xcorr.max(), 1, decimal=5)
+    assert_array_equal(max_index, np.array(arr1.shape) / 2)
