@@ -1,13 +1,17 @@
+import functools
+from math import ceil
+import numbers
+
 import scipy.stats
 import numpy as np
-from math import ceil
-from .. import img_as_float
-from ._denoise_cy import _denoise_bilateral, _denoise_tv_bregman
-from .._shared.utils import warn
 import pywt
-import skimage.color as color
-from skimage.color.colorconv import ycbcr_from_rgb
-import numbers
+
+from ..util.dtype import img_as_float
+from .._shared import utils
+from .._shared.utils import _supported_float_type, warn
+from ._denoise_cy import _denoise_bilateral, _denoise_tv_bregman
+from .. import color
+from ..color.colorconv import ycbcr_from_rgb
 
 
 def _gaussian_weight(array, sigma_squared, *, dtype=float):
@@ -28,7 +32,7 @@ def _gaussian_weight(array, sigma_squared, *, dtype=float):
     gaussian : ndarray
         The input array filtered by the Gaussian.
     """
-    return np.exp(-0.5 * (array ** 2  / sigma_squared), dtype=dtype)
+    return np.exp(-0.5 * (array ** 2 / sigma_squared), dtype=dtype)
 
 
 def _compute_color_lut(bins, sigma, max_value, *, dtype=float):
@@ -37,7 +41,7 @@ def _compute_color_lut(bins, sigma, max_value, *, dtype=float):
 
     Parameters
     ----------
-     bins : int
+    bins : int
         Number of discrete values for Gaussian weights of color filtering.
         A larger value results in improved accuracy.
     sigma : float
@@ -88,8 +92,11 @@ def _compute_spatial_lut(win_size, sigma, *, dtype=float):
     return _gaussian_weight(distances, sigma**2, dtype=dtype).ravel()
 
 
+@utils.channel_as_last_axis()
+@utils.deprecate_multichannel_kwarg(multichannel_position=7)
 def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
-                      bins=10000, mode='constant', cval=0, multichannel=False):
+                      bins=10000, mode='constant', cval=0, multichannel=False,
+                      *, channel_axis=None):
     """Denoise image using bilateral filter.
 
     Parameters
@@ -103,10 +110,8 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     sigma_color : float
         Standard deviation for grayvalue/color distance (radiometric
         similarity). A larger value results in averaging of pixels with larger
-        radiometric differences. Note, that the image will be converted using
-        the `img_as_float` function and thus the standard deviation is in
-        respect to the range ``[0, 1]``. If the value is ``None`` the standard
-        deviation of the ``image`` will be used.
+        radiometric differences. If ``None``, the standard deviation of
+        ``image`` will be used.
     sigma_spatial : float
         Standard deviation for range distance. A larger value results in
         averaging of pixels with larger spatial differences.
@@ -121,7 +126,15 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
         the image boundaries.
     multichannel : bool
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension.
+        channels or another spatial dimension. This argument is deprecated:
+        specify `channel_axis` instead.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Returns
     -------
@@ -141,6 +154,14 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     Euclidean distance between two color values and a certain standard
     deviation (`sigma_color`).
 
+    Note that, if the image is of any `int` dtype, ``image`` will be
+    converted using the `img_as_float` function and thus the standard
+    deviation (`sigma_color`) will be in range ``[0, 1]``.
+
+    For more information on scikit-image's data type conversions and how
+    images are rescaled in these conversions,
+    see: https://scikit-image.org/docs/stable/user_guide/data_types.html.
+
     References
     ----------
     .. [1] C. Tomasi and R. Manduchi. "Bilateral Filtering for Gray and Color
@@ -152,12 +173,13 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     >>> from skimage import data, img_as_float
     >>> astro = img_as_float(data.astronaut())
     >>> astro = astro[220:300, 220:320]
-    >>> noisy = astro + 0.6 * astro.std() * np.random.random(astro.shape)
+    >>> rng = np.random.default_rng()
+    >>> noisy = astro + 0.6 * astro.std() * rng.random(astro.shape)
     >>> noisy = np.clip(noisy, 0, 1)
     >>> denoised = denoise_bilateral(noisy, sigma_color=0.05, sigma_spatial=15,
-    ...                              multichannel=True)
+    ...                              channel_axis=-1)
     """
-    if multichannel:
+    if channel_axis is not None:
         if image.ndim != 3:
             if image.ndim == 2:
                 raise ValueError("Use ``multichannel=False`` for 2D grayscale "
@@ -165,29 +187,28 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
                                  "must be multiple color channels not another "
                                  "spatial dimension.")
             else:
-                raise ValueError("Bilateral filter is only implemented for "
-                                 "2D grayscale images (image.ndim == 2) and "
-                                 "2D multichannel (image.ndim == 3) images, "
-                                 "but the input image has {0} dimensions. "
-                                 "".format(image.ndim))
+                raise ValueError(f'Bilateral filter is only implemented for '
+                                 f'2D grayscale images (image.ndim == 2) and '
+                                 f'2D multichannel (image.ndim == 3) images, '
+                                 f'but the input image has {image.ndim} dimensions.')
         elif image.shape[2] not in (3, 4):
             if image.shape[2] > 4:
-                msg = ("The last axis of the input image is interpreted as "
-                       "channels. Input image with shape {0} has {1} channels "
-                       "in last axis. ``denoise_bilateral`` is implemented "
-                       "for 2D grayscale and color images only")
-                warn(msg.format(image.shape, image.shape[2]))
+                msg = f'The last axis of the input image is ' \
+                      f'interpreted as channels. Input image with '\
+                      f'shape {image.shape} has {image.shape[2]} channels '\
+                      f'in last axis. ``denoise_bilateral``is implemented ' \
+                      f'for 2D grayscale and color images only.'
+                warn(msg)
             else:
-                msg = "Input image must be grayscale, RGB, or RGBA; " \
-                      "but has shape {0}."
-                warn(msg.format(image.shape))
+                msg = f'Input image must be grayscale, RGB, or RGBA; ' \
+                      f'but has shape {image.shape}.'
+                warn(msg)
     else:
         if image.ndim > 2:
-            raise ValueError("Bilateral filter is not implemented for "
-                             "grayscale images of 3 or more dimensions, "
-                             "but input image has {0} dimension. Use "
-                             "``multichannel=True`` for 2-D RGB "
-                             "images.".format(image.shape))
+            raise ValueError(f'Bilateral filter is not implemented for '
+                             f'grayscale images of 3 or more dimensions, '
+                             f'but input image has {image.shape} shape. Use '
+                             f'``channel_axis=-1`` for 2D RGB images.')
 
     if win_size is None:
         win_size = max(5, 2 * int(ceil(3 * sigma_spatial)) + 1)
@@ -201,11 +222,7 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     # if image.max() is 0, then dist_scale can have an unverified value
     # and color_lut[<int>(dist * dist_scale)] may cause a segmentation fault
     # so we verify we have a positive image and that the max is not 0.0.
-    if min_value < 0.0:
-        raise ValueError("Image must contain only positive values")
 
-    if max_value == 0.0:
-        raise ValueError("The maximum value found in the image was 0.")
 
     image = np.atleast_3d(img_as_float(image))
     image = np.ascontiguousarray(image)
@@ -215,7 +232,8 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     color_lut = _compute_color_lut(bins, sigma_color, max_value,
                                    dtype=image.dtype)
 
-    range_lut = _compute_spatial_lut(win_size, sigma_spatial, dtype=image.dtype)
+    range_lut = _compute_spatial_lut(win_size, sigma_spatial,
+                                     dtype=image.dtype)
 
     out = np.empty(image.shape, dtype=image.dtype)
 
@@ -227,13 +245,25 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     # where needed within Cython.
     empty_dims = np.empty(dims, dtype=image.dtype)
 
-    return _denoise_bilateral(image, image.max(), win_size, sigma_color,
-                              sigma_spatial, bins, mode, cval, color_lut,
-                              range_lut, empty_dims, out)
+    if min_value < 0:
+        image = image - min_value
+        max_value -= min_value
+    _denoise_bilateral(image, max_value, win_size, sigma_color, sigma_spatial,
+                       bins, mode, cval, color_lut, range_lut, empty_dims, out)
+    # need to drop the added channels axis for grayscale images
+    out = np.squeeze(out)
+    if min_value < 0:
+        out += min_value
+    return out
 
 
-def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True,
-                       *, multichannel=False):
+@utils.channel_as_last_axis()
+@utils.deprecate_multichannel_kwarg()
+@utils.deprecate_kwarg({'max_iter': 'max_num_iter'}, removed_version="1.0",
+                       deprecated_version="0.19")
+def denoise_tv_bregman(image, weight=5.0, max_num_iter=100, eps=1e-3,
+                       isotropic=True, *, channel_axis=None,
+                       multichannel=False):
     """Perform total-variation denoising using split-Bregman optimization.
 
     Total-variation denoising (also know as total-variation regularization)
@@ -255,14 +285,22 @@ def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True,
 
             SUM((u(n) - u(n-1))**2) < eps
 
-    max_iter : int, optional
+    max_num_iter : int, optional
         Maximal number of iterations used for the optimization.
     isotropic : boolean, optional
         Switch between isotropic and anisotropic TV denoising.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
     multichannel : bool, optional
         Apply total-variation denoising separately for each channel. This
         option should be true for color images, otherwise the denoising is
-        also applied in the channels dimension.
+        also applied in the channels dimension. This argument is deprecated:
+        specify `channel_axis` instead.
 
     Returns
     -------
@@ -291,28 +329,29 @@ def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True,
 
     out = np.zeros(shape_ext, image.dtype)
 
-    if multichannel:
+    if channel_axis is not None:
         channel_out = np.zeros(shape_ext[:2] + (1,), dtype=out.dtype)
         for c in range(image.shape[-1]):
             # the algorithm below expects 3 dimensions to always be present.
-            # slicing the array in this fashion preserves the channel dimension for us
+            # slicing the array in this fashion preserves the channel dimension
+            # for us
             channel_in = np.ascontiguousarray(image[..., c:c+1])
 
             _denoise_tv_bregman(channel_in, image.dtype.type(weight),
-                                max_iter, eps, isotropic, channel_out)
+                                max_num_iter, eps, isotropic, channel_out)
 
             out[..., c] = channel_out[..., 0]
 
     else:
         image = np.ascontiguousarray(image)
 
-        _denoise_tv_bregman(image, image.dtype.type(weight), max_iter, eps,
+        _denoise_tv_bregman(image, image.dtype.type(weight), max_num_iter, eps,
                             isotropic, out)
 
     return np.squeeze(out[1:-1, 1:-1])
 
 
-def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
+def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, max_num_iter=200):
     """Perform total-variation denoising on n-dimensional images.
 
     Parameters
@@ -328,7 +367,7 @@ def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
 
             (E_(n-1) - E_n) < eps * E_0
 
-    n_iter_max : int, optional
+    max_num_iter : int, optional
         Maximal number of iterations used for the optimization.
 
     Returns
@@ -346,7 +385,7 @@ def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
     g = np.zeros_like(p)
     d = np.zeros_like(image)
     i = 0
-    while i < n_iter_max:
+    while i < max_num_iter:
         if i > 0:
             # d will be the (negative) divergence of p
             d = -p.sum(0)
@@ -393,8 +432,11 @@ def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
     return out
 
 
-def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
-                         multichannel=False):
+@utils.deprecate_kwarg({'n_iter_max': 'max_num_iter'}, removed_version="1.0",
+                       deprecated_version="0.19.2")
+@utils.deprecate_multichannel_kwarg(multichannel_position=4)
+def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, max_num_iter=200,
+                         multichannel=False, *, channel_axis=None):
     """Perform total-variation denoising on n-dimensional images.
 
     Parameters
@@ -412,12 +454,20 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
 
             (E_(n-1) - E_n) < eps * E_0
 
-    n_iter_max : int, optional
+    max_num_iter : int, optional
         Maximal number of iterations used for the optimization.
     multichannel : bool, optional
         Apply total-variation denoising separately for each channel. This
         option should be true for color images, otherwise the denoising is
-        also applied in the channels dimension.
+        also applied in the channels dimension. This argument is deprecated:
+        specify `channel_axis` instead.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Returns
     -------
@@ -452,7 +502,8 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
 
     >>> from skimage import color, data
     >>> img = color.rgb2gray(data.astronaut())[:50, :50]
-    >>> img += 0.5 * img.std() * np.random.randn(*img.shape)
+    >>> rng = np.random.default_rng()
+    >>> img += 0.5 * img.std() * rng.standard_normal(img.shape)
     >>> denoised_img = denoise_tv_chambolle(img, weight=60)
 
     3D example on synthetic data:
@@ -460,7 +511,8 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
     >>> x, y, z = np.ogrid[0:20, 0:20, 0:20]
     >>> mask = (x - 22)**2 + (y - 20)**2 + (z - 17)**2 < 8**2
     >>> mask = mask.astype(float)
-    >>> mask += 0.2*np.random.randn(*mask.shape)
+    >>> rng = np.random.default_rng()
+    >>> mask += 0.2 * rng.standard_normal(mask.shape)
     >>> res = denoise_tv_chambolle(mask, weight=100)
 
     """
@@ -469,13 +521,19 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
     if not im_type.kind == 'f':
         image = img_as_float(image)
 
-    if multichannel:
+    # enforce float16->float32 and float128->float64
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
+
+    if channel_axis is not None:
+        channel_axis = channel_axis % image.ndim
+        _at = functools.partial(utils.slice_at_axis, axis=channel_axis)
         out = np.zeros_like(image)
-        for c in range(image.shape[-1]):
-            out[..., c] = _denoise_tv_chambolle_nd(image[..., c], weight, eps,
-                                                   n_iter_max)
+        for c in range(image.shape[channel_axis]):
+            out[_at(c)] = _denoise_tv_chambolle_nd(image[_at(c)], weight, eps,
+                                                   max_num_iter)
     else:
-        out = _denoise_tv_chambolle_nd(image, weight, eps, n_iter_max)
+        out = _denoise_tv_chambolle_nd(image, weight, eps, max_num_iter)
     return out
 
 
@@ -579,9 +637,10 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
     """
     wavelet = pywt.Wavelet(wavelet)
     if not wavelet.orthogonal:
-        warn(("Wavelet thresholding was designed for use with orthogonal "
-              "wavelets. For nonorthogonal wavelets such as {}, results are "
-              "likely to be suboptimal.").format(wavelet.name))
+        warn(f'Wavelet thresholding was designed for '
+             f'use with orthogonal wavelets. For nonorthogonal '
+             f'wavelets such as {wavelet.name},results are '
+             f'likely to be suboptimal.')
 
     # original_extent is used to workaround PyWavelets issue #80
     # odd-sized input results in an image with 1 extra sample after waverecn
@@ -590,7 +649,6 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
     # Determine the number of wavelet decomposition levels
     if wavelet_levels is None:
         # Determine the maximum number of possible levels for image
-        dlen = wavelet.dec_len
         wavelet_levels = pywt.dwtn_max_level(image.shape, wavelet)
 
         # Skip coarsest wavelet scales (see Notes in docstring).
@@ -606,8 +664,8 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
         sigma = _sigma_est_dwt(detail_coeffs, distribution='Gaussian')
 
     if method is not None and threshold is not None:
-        warn(("Thresholding method {} selected.  The user-specified threshold "
-              "will be ignored.").format(method))
+        warn(f'Thresholding method {method} selected. The '
+             f'user-specified threshold will be ignored.')
 
     if threshold is None:
         var = sigma**2
@@ -622,7 +680,7 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
             # The VisuShrink thresholds from [2]_ in docstring
             threshold = _universal_thresh(image, sigma)
         else:
-            raise ValueError("Unrecognized method: {}".format(method))
+            raise ValueError(f'Unrecognized method: {method}')
 
     if np.isscalar(threshold):
         # A single threshold for all coefficient arrays
@@ -645,6 +703,7 @@ def _scale_sigma_and_image_consistently(image, sigma, multichannel,
     """If the ``image`` is rescaled, also rescale ``sigma`` consistently.
 
     Images that are not floating point will be rescaled via ``img_as_float``.
+    Half-precision images will be promoted to single precision.
     """
     if multichannel:
         if isinstance(sigma, numbers.Number) or sigma is None:
@@ -666,6 +725,8 @@ def _scale_sigma_and_image_consistently(image, sigma, multichannel,
                          for s in sigma]
             elif sigma is not None:
                 sigma *= scale_factor
+    elif image.dtype == np.float16:
+        image = image.astype(np.float32)
     return image, sigma
 
 
@@ -691,10 +752,12 @@ def _rescale_sigma_rgb2ycbcr(sigmas):
     return sigmas
 
 
+@utils.channel_as_last_axis()
+@utils.deprecate_multichannel_kwarg(multichannel_position=5)
 def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
                     wavelet_levels=None, multichannel=False,
                     convert2ycbcr=False, method='BayesShrink',
-                    rescale_sigma=True):
+                    rescale_sigma=True, *, channel_axis=None):
     """Perform wavelet denoising on an image.
 
     Parameters
@@ -720,7 +783,8 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
         three less than the maximum number of possible decomposition levels.
     multichannel : bool, optional
         Apply wavelet denoising separately for each channel (where channels
-        correspond to the final axis of the array).
+        correspond to the final axis of the array). This argument is
+        deprecated: specify `channel_axis` instead.
     convert2ycbcr : bool, optional
         If True and multichannel True, do the wavelet denoising in the YCbCr
         colorspace instead of the RGB color space. This typically results in
@@ -735,6 +799,13 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
 
         .. versionadded:: 0.16
            ``rescale_sigma`` was introduced in 0.16
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Returns
     -------
@@ -796,15 +867,16 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
     >>> from skimage import color, data
     >>> img = img_as_float(data.astronaut())
     >>> img = color.rgb2gray(img)
-    >>> img += 0.1 * np.random.randn(*img.shape)
+    >>> rng = np.random.default_rng()
+    >>> img += 0.1 * rng.standard_normal(img.shape)
     >>> img = np.clip(img, 0, 1)
     >>> denoised_img = denoise_wavelet(img, sigma=0.1, rescale_sigma=True)
 
     """
+    multichannel = channel_axis is not None
     if method not in ["BayesShrink", "VisuShrink"]:
-        raise ValueError(
-            ('Invalid method: {}. The currently supported methods are '
-             '"BayesShrink" and "VisuShrink"').format(method))
+        raise ValueError(f'Invalid method: {method}. The currently supported '
+                         f'methods are "BayesShrink" and "VisuShrink".')
 
     # floating-point inputs are not rescaled, so don't clip their output.
     clip_output = image.dtype.kind != 'f'
@@ -863,7 +935,9 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
     return out
 
 
-def estimate_sigma(image, average_sigmas=False, multichannel=False):
+@utils.deprecate_multichannel_kwarg(multichannel_position=2)
+def estimate_sigma(image, average_sigmas=False, multichannel=False, *,
+                   channel_axis=None):
     """
     Robust wavelet-based estimator of the (Gaussian) noise standard deviation.
 
@@ -875,7 +949,15 @@ def estimate_sigma(image, average_sigmas=False, multichannel=False):
         If true, average the channel estimates of `sigma`.  Otherwise return
         a list of sigmas corresponding to each channel.
     multichannel : bool
-        Estimate sigma separately for each channel.
+        Estimate sigma separately for each channel. This argument is
+        deprecated: specify `channel_axis` instead.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Returns
     -------
@@ -903,21 +985,24 @@ def estimate_sigma(image, average_sigmas=False, multichannel=False):
     >>> from skimage import img_as_float
     >>> img = img_as_float(skimage.data.camera())
     >>> sigma = 0.1
-    >>> img = img + sigma * np.random.standard_normal(img.shape)
-    >>> sigma_hat = estimate_sigma(img, multichannel=False)
+    >>> rng = np.random.default_rng()
+    >>> img = img + sigma * rng.standard_normal(img.shape)
+    >>> sigma_hat = estimate_sigma(img, channel_axis=None)
     """
-    if multichannel:
-        nchannels = image.shape[-1]
+    if channel_axis is not None:
+        channel_axis = channel_axis % image.ndim
+        _at = functools.partial(utils.slice_at_axis, axis=channel_axis)
+        nchannels = image.shape[channel_axis]
         sigmas = [estimate_sigma(
-            image[..., c], multichannel=False) for c in range(nchannels)]
+            image[_at(c)], channel_axis=None) for c in range(nchannels)]
         if average_sigmas:
             sigmas = np.mean(sigmas)
         return sigmas
     elif image.shape[-1] <= 4:
-        msg = ("image is size {0} on the last axis, but multichannel is "
-               "False.  If this is a color image, please set multichannel "
-               "to True for proper noise estimation.")
-        warn(msg.format(image.shape[-1]))
+        msg = f'image is size {image.shape[-1]} on the last axis, '\
+              f'but channel_axis is None. If this is a color image, '\
+              f'please set channel_axis=-1 for proper noise estimation.'
+        warn(msg)
     coeffs = pywt.dwtn(image, wavelet='db2')
     detail_coeffs = coeffs['d' * image.ndim]
     return _sigma_est_dwt(detail_coeffs, distribution='Gaussian')
