@@ -11,7 +11,6 @@ from ._watershed import watershed
 from ._chan_vese import chan_vese
 from .morphsnakes import (morphological_geodesic_active_contour,
                           morphological_chan_vese, inverse_gaussian_gradient,
-                          circle_level_set,
                           disk_level_set, checkerboard_level_set)
 from ..morphology import flood, flood_fill
 
@@ -33,7 +32,6 @@ __all__ = [
     'morphological_geodesic_active_contour',
     'morphological_chan_vese',
     'inverse_gaussian_gradient',
-    'circle_level_set',
     'disk_level_set',
     'checkerboard_level_set',
     'flood',
