@@ -1,19 +1,19 @@
 import itertools
-import pytest
 
 import numpy as np
+import pytest
+
+from skimage._shared._dependency_checks import has_mpl
+from skimage._shared._warnings import expected_warnings
+from skimage._shared.testing import test_parallel
+from skimage._shared.utils import _supported_float_type, convert_to_float
 from skimage.data import shepp_logan_phantom
 from skimage.transform import radon, iradon, iradon_sart, rescale
 
-from skimage._shared.utils import convert_to_float
-from skimage._shared import testing
-from skimage._shared.testing import test_parallel
-from skimage._shared._warnings import expected_warnings
-
 
 PHANTOM = shepp_logan_phantom()[::2, ::2]
 PHANTOM = rescale(PHANTOM, 0.5, order=1,
-                  mode='constant', anti_aliasing=False, multichannel=False)
+                  mode='constant', anti_aliasing=False, channel_axis=None)
 
 
 def _debug_plot(original, result, sinogram=None):
@@ -72,24 +72,27 @@ def check_radon_center(shape, circle, dtype, preserve_range):
     theta = np.linspace(0., 180., max(shape), endpoint=False)
     sinogram = radon(image, theta=theta, circle=circle,
                      preserve_range=preserve_range)
+    assert sinogram.dtype == _supported_float_type(sinogram.dtype)
     # The sinogram should be a straight, horizontal line
     sinogram_max = np.argmax(sinogram, axis=0)
     print(sinogram_max)
     assert np.std(sinogram_max) < 1e-6
 
 
-@testing.parametrize("shape", [(16, 16), (17, 17)])
-@testing.parametrize("circle", [False, True])
-@testing.parametrize("dtype", [np.float64, np.float32, np.uint8, bool])
-@testing.parametrize("preserve_range", [False, True])
+@pytest.mark.parametrize("shape", [(16, 16), (17, 17)])
+@pytest.mark.parametrize("circle", [False, True])
+@pytest.mark.parametrize(
+    "dtype", [np.float64, np.float32, np.float16, np.uint8, bool]
+)
+@pytest.mark.parametrize("preserve_range", [False, True])
 def test_radon_center(shape, circle, dtype, preserve_range):
     check_radon_center(shape, circle, dtype, preserve_range)
 
 
-@testing.parametrize("shape", [(32, 16), (33, 17)])
-@testing.parametrize("circle", [False])
-@testing.parametrize("dtype", [np.float64, np.float32, np.uint8, bool])
-@testing.parametrize("preserve_range", [False, True])
+@pytest.mark.parametrize("shape", [(32, 16), (33, 17)])
+@pytest.mark.parametrize("circle", [False])
+@pytest.mark.parametrize("dtype", [np.float64, np.float32, np.uint8, bool])
+@pytest.mark.parametrize("preserve_range", [False, True])
 def test_radon_center_rectangular(shape, circle, dtype, preserve_range):
     check_radon_center(shape, circle, dtype, preserve_range)
 
@@ -115,7 +118,7 @@ def check_iradon_center(size, theta, circle):
                                      circle=circle)
     print('rms deviance:',
           np.sqrt(np.mean((reconstruction_opposite - reconstruction)**2)))
-    if debug:
+    if debug and has_mpl:
         import matplotlib.pyplot as plt
         imkwargs = dict(cmap='gray', interpolation='nearest')
         plt.figure()
@@ -137,10 +140,12 @@ thetas_for_test_iradon_center = [0, 90]
 circles_for_test_iradon_center = [False, True]
 
 
-@testing.parametrize("size, theta, circle",
-                     itertools.product(sizes_for_test_iradon_center,
-                                       thetas_for_test_iradon_center,
-                                       circles_for_test_iradon_center))
+@pytest.mark.parametrize(
+    "size, theta, circle",
+    itertools.product(sizes_for_test_iradon_center,
+                      thetas_for_test_iradon_center,
+                      circles_for_test_iradon_center)
+)
 def test_iradon_center(size, theta, circle):
     check_iradon_center(size, theta, circle)
 
@@ -152,7 +157,7 @@ def check_radon_iradon(interpolation_type, filter_type):
                            interpolation=interpolation_type, circle=False)
     delta = np.mean(np.abs(image - reconstructed))
     print('\n\tmean error:', delta)
-    if debug:
+    if debug and has_mpl:
         _debug_plot(image, reconstructed)
     if filter_type in ('ramp', 'shepp-logan'):
         if interpolation_type == 'nearest':
@@ -172,21 +177,13 @@ radon_iradon_inputs = list(itertools.product(interpolation_types,
 radon_iradon_inputs.append(('cubic', 'shepp-logan'))
 
 
-@testing.parametrize("interpolation_type, filter_type",
-                     radon_iradon_inputs)
+@pytest.mark.parametrize(
+    "interpolation_type, filter_type", radon_iradon_inputs
+)
 def test_radon_iradon(interpolation_type, filter_type):
     check_radon_iradon(interpolation_type, filter_type)
 
 
-@pytest.mark.parametrize("filter_type", filter_types)
-def test_iradon_new_signature(filter_type):
-    image = PHANTOM
-    sinogram = radon(image, circle=False)
-    with pytest.warns(FutureWarning):
-        assert np.array_equal(iradon(sinogram, filter=filter_type),
-                              iradon(sinogram, filter_name=filter_type))
-
-
 def test_iradon_angles():
     """
     Test with different number of projections
@@ -223,7 +220,7 @@ def check_radon_iradon_minimal(shape, slices):
     sinogram = radon(image, theta, circle=False)
     reconstructed = iradon(sinogram, theta, circle=False)
     print('\n\tMaximum deviation:', np.max(np.abs(image - reconstructed)))
-    if debug:
+    if debug and has_mpl:
         _debug_plot(image, reconstructed, sinogram)
     if image.sum() == 1:
         assert (np.unravel_index(np.argmax(reconstructed), image.shape)
@@ -247,8 +244,10 @@ def generate_test_data_for_radon_iradon_minimal(shapes):
                                           for shape in shapes])
 
 
-@testing.parametrize("shape, coordinate",
-                     generate_test_data_for_radon_iradon_minimal(shapes))
+@pytest.mark.parametrize(
+    "shape, coordinate",
+    generate_test_data_for_radon_iradon_minimal(shapes)
+)
 def test_radon_iradon_minimal(shape, coordinate):
     check_radon_iradon_minimal(shape, coordinate)
 
@@ -257,7 +256,7 @@ def test_reconstruct_with_wrong_angles():
     a = np.zeros((3, 3))
     p = radon(a, theta=[0, 1, 2], circle=False)
     iradon(p, theta=[0, 1, 2], circle=False)
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         iradon(p, theta=[0, 1, 2, 3])
 
 
@@ -319,7 +318,7 @@ def check_sinogram_circle_to_square(size):
             argmax_shape(sinogram_circle_to_square))
 
 
-@testing.parametrize("size", (50, 51))
+@pytest.mark.parametrize("size", (50, 51))
 def test_sinogram_circle_to_square(size):
     check_sinogram_circle_to_square(size)
 
@@ -361,9 +360,10 @@ output_sizes = (None,
                 97)
 
 
-@testing.parametrize("shape, interpolation, output_size",
-                     itertools.product(shapes_radon_iradon_circle,
-                                       interpolations, output_sizes))
+@pytest.mark.parametrize(
+    "shape, interpolation, output_size",
+    itertools.product(shapes_radon_iradon_circle, interpolations, output_sizes)
+)
 def test_radon_iradon_circle(shape, interpolation, output_size):
     check_radon_iradon_circle(interpolation, shape, output_size)
 
@@ -386,7 +386,7 @@ def test_iradon_sart():
     debug = False
 
     image = rescale(PHANTOM, 0.8, mode='reflect',
-                    multichannel=False, anti_aliasing=False)
+                    channel_axis=None, anti_aliasing=False)
     theta_ordered = np.linspace(0., 180., image.shape[0], endpoint=False)
     theta_missing_wedge = np.linspace(0., 150., image.shape[0], endpoint=True)
     for theta, error_factor in ((theta_ordered, 1.),
@@ -394,7 +394,7 @@ def test_iradon_sart():
         sinogram = radon(image, theta, circle=True)
         reconstructed = iradon_sart(sinogram, theta)
 
-        if debug:
+        if debug and has_mpl:
             from matplotlib import pyplot as plt
             plt.figure()
             plt.subplot(221)
@@ -427,7 +427,7 @@ def test_iradon_sart():
                                       for i in range(sinogram.shape[1])]).T
         reconstructed = iradon_sart(sinogram_shifted, theta,
                                     projection_shifts=shifts)
-        if debug:
+        if debug and has_mpl:
             from matplotlib import pyplot as plt
             plt.figure()
             plt.subplot(221)
@@ -489,5 +489,5 @@ def test_iradon_sart_dtype(dtype):
 def test_iradon_sart_wrong_dtype():
     sinogram = np.zeros((16, 1))
 
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         iradon_sart(sinogram, dtype=int)
