@@ -15,14 +15,7 @@ from .unique import unique_rows
 from ._invert import invert
 from ._montage import montage
 from ._map_array import map_array
-
-
-@functools.wraps(np.pad)
-def pad(*args, **kwargs):
-    warnings.warn("skimage.util.pad is deprecated and will be removed in "
-                  "version 0.19. Please use numpy.pad instead.",
-                  FutureWarning, stacklevel=2)
-    return np.pad(*args, **kwargs)
+from ._label import label_points
 
 
 __all__ = ['img_as_float32',
@@ -35,7 +28,6 @@ __all__ = ['img_as_float32',
            'dtype_limits',
            'view_as_blocks',
            'view_as_windows',
-           'pad',
            'crop',
            'compare_images',
            'map_array',
@@ -46,4 +38,5 @@ __all__ = ['img_as_float32',
            'apply_parallel',
            'invert',
            'unique_rows',
+           'label_points',
            ]
