@@ -130,12 +130,12 @@ def _scale(a, n, m, copy=True):
     if n > m and a.max() < 2 ** m:
         mnew = int(np.ceil(m / 2) * 2)
         if mnew > m:
-            dtype = "int{}".format(mnew)
+            dtype = f'int{mnew}'
         else:
-            dtype = "uint{}".format(mnew)
+            dtype = f'uint{mnew}'
         n = int(np.ceil(n / 2) * 2)
-        warn("Downcasting {} to {} without scaling because max "
-             "value {} fits in {}".format(a.dtype, dtype, a.max(), dtype),
+        warn(f'Downcasting {a.dtype} to {dtype} without scaling because max '
+             f'value {a.max()} fits in {dtype}',
              stacklevel=3)
         return a.astype(_dtype_bits(kind, m))
     elif n == m:
@@ -205,7 +205,7 @@ def _convert(image, dtype, force_copy=False, uniform=False):
         rounded to the nearest integers, which minimizes back and forth
         conversion errors.
 
-    .. versionchanged :: 0.15
+    .. versionchanged:: 0.15
         ``_convert`` no longer warns about possible precision or sign
         information loss. See discussions on these warnings at:
         https://github.com/scikit-image/scikit-image/issues/2602
@@ -252,8 +252,8 @@ def _convert(image, dtype, force_copy=False, uniform=False):
         return image
 
     if not (dtype_in in _supported_types and dtype_out in _supported_types):
-        raise ValueError("Can not convert from {} to {}."
-                         .format(dtypeobj_in, dtypeobj_out))
+        raise ValueError(f'Cannot convert from {dtypeobj_in} to '
+                         f'{dtypeobj_out}.')
 
     if kind_in in 'ui':
         imin_in = np.iinfo(dtype_in).min
@@ -321,6 +321,16 @@ def _convert(image, dtype, force_copy=False, uniform=False):
             # DirectX uses this conversion also for signed ints
             # if imin_in:
             #     np.maximum(image, -1.0, out=image)
+        elif kind_in == 'i':
+            # From DirectX conversions:
+            # The most negative value maps to -1.0f
+            # Every other value is converted to a float (call it c)
+            # and then result = c * (1.0f / (2⁽ⁿ⁻¹⁾-1)).
+
+            image = np.multiply(image, 1. / imax_in,
+                                dtype=computation_type)
+            np.maximum(image, -1.0, out=image)
+
         else:
             image = np.add(image, 0.5, dtype=computation_type)
             image *= 2 / (imax_in - imin_in)
@@ -357,7 +367,7 @@ def _convert(image, dtype, force_copy=False, uniform=False):
 
 def convert(image, dtype, force_copy=False, uniform=False):
     warn("The use of this function is discouraged as its behavior may change "
-         "dramatically in scikit-image 1.0. This function will be removed"
+         "dramatically in scikit-image 1.0. This function will be removed "
          "in scikit-image 1.0.", FutureWarning, stacklevel=2)
     return _convert(image=image, dtype=dtype,
                     force_copy=force_copy, uniform=uniform)
