@@ -1,7 +1,7 @@
 import numpy as np
 
 from skimage._shared import testing
-from skimage._shared.testing import assert_equal, assert_warns
+from skimage._shared.testing import assert_equal
 from skimage.util.shape import view_as_blocks, view_as_windows
 
 
