@@ -1,6 +1,11 @@
 import os
 
-from skimage import data, img_as_float, io, img_as_uint
+from numpy.testing import assert_almost_equal, assert_equal
+import pytest
+
+pytest.importorskip("matplotlib")
+
+from skimage import data, img_as_float, io, img_as_ubyte
 
 from skimage.viewer import ImageViewer
 from skimage.viewer.qt import QtWidgets, QtCore, has_qt
@@ -8,10 +13,6 @@ from skimage.viewer.widgets import (
     Slider, OKCancelButtons, SaveButtons, ComboBox, CheckBox, Text)
 from skimage.viewer.plugins.base import Plugin
 
-from skimage._shared._warnings import expected_warnings
-from skimage._shared import testing
-from skimage._shared.testing import assert_almost_equal, assert_equal
-
 
 def get_image_viewer():
     image = data.coins()
@@ -20,7 +21,7 @@ def get_image_viewer():
     return viewer
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_check_box():
     viewer = get_image_viewer()
     cb = CheckBox('hello', value=True, alignment='left')
@@ -35,7 +36,7 @@ def test_check_box():
     assert_equal(cb.val, False)
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_combo_box():
     viewer = get_image_viewer()
     cb = ComboBox('hello', ('a', 'b', 'c'))
@@ -48,7 +49,7 @@ def test_combo_box():
     assert_equal(cb.index, 2)
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_text_widget():
     viewer = get_image_viewer()
     txt = Text('hello', 'hello, world!')
@@ -59,7 +60,7 @@ def test_text_widget():
     assert_equal(str(txt.text), 'goodbye, world!')
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_slider_int():
     viewer = get_image_viewer()
     sld = Slider('radius', 2, 10, value_type='int')
@@ -73,7 +74,7 @@ def test_slider_int():
     assert_equal(sld.val, 5)
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_slider_float():
     viewer = get_image_viewer()
     sld = Slider('alpha', 2.1, 3.1, value=2.1, value_type='float',
@@ -88,8 +89,7 @@ def test_slider_float():
     assert_almost_equal(sld.val, 2.5, 2)
 
 
-@testing.skipif(True, reason="Can't automatically close window. See #3081.")
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_save_buttons():
     viewer = get_image_viewer()
     sv = SaveButtons()
@@ -108,13 +108,11 @@ def test_save_buttons():
 
     # call the save functions directly
     sv.save_to_stack()
-    with expected_warnings(['precision loss']):
-        sv.save_to_file(filename)
+    sv.save_to_file(filename)
 
-    img = data.imread(filename)
+    img = io.imread(filename)
 
-    with expected_warnings(['precision loss']):
-        assert_almost_equal(img, img_as_uint(viewer.image))
+    assert_almost_equal(img, img_as_ubyte(viewer.image))
 
     img = io.pop()
     assert_almost_equal(img, viewer.image)
@@ -122,7 +120,7 @@ def test_save_buttons():
     os.remove(filename)
 
 
-@testing.skipif(not has_qt, reason="Qt not installed")
+@pytest.mark.skipif(not has_qt, reason="Qt not installed")
 def test_ok_buttons():
     viewer = get_image_viewer()
     ok = OKCancelButtons()
