@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # Fail on non-zero exit and echo the commands
-set -ev
+set -evx
 
 mkdir -p $MPL_DIR
 touch $MPL_DIR/matplotlibrc
@@ -9,10 +9,21 @@ touch $MPL_DIR/matplotlibrc
 python -m pip list
 tools/build_versions.py
 
+TEST_ARGS="--doctest-modules --cov=skimage"
+
+if [[ ${WITHOUT_POOCH} == "1" ]]; then
+  # remove pooch (previously installed via requirements/test.txt)
+  pip uninstall pooch -y
+fi
+
 # When installing from sdist
 # We can't run it in the git directory since there is a folder called `skimage`
 # in there. pytest will crawl that instead of the module we installed and want to test
-(cd .. && pytest $TEST_ARGS --pyargs skimage)
+if [[ ${INSTALL_FROM_SDIST} == "1" ]]; then
+  (cd .. && python -c "import skimage; skimage.test()")
+else
+  (cd .. && pytest $TEST_ARGS --pyargs skimage)
+fi
 
 flake8 --exit-zero --exclude=test_* skimage doc/examples viewer_examples
 
